import barycorrpy
import configparser
import os
from astroquery.mast import Catalogs
import bary
DIRNAME = os.path.dirname(__file__)
PATH_TARGETS = os.path.join(DIRNAME,'data/target_files')

class Target(object):
    """
    Simple target class. Capable of querying SIMBAD. Can calculate barycentric corrections.
    
    EXAMPLE:
        H = HPFSpectrum(fitsfiles[1])
        H.plot_order(14,deblazed=True)
        T = Target('G 9-40')
        T.calc_barycentric_velocity(H.jd_midpoint,'McDonald Observatory')
        T = Target('G 9-40')
    """
    
    def __init__(self,name,config_folder=PATH_TARGETS,verbose=False,obsname='Kitt Peak National Observatory'):
        self.config_folder = config_folder
        self.config_filename = self.config_folder + os.sep + name + '.config'
        if name=='Teegarden':
            name = "Teegarden's star"
        if name=='HR8926-4':
            name = 'GL87'
        if name=='GJ_1151':
            name = 'GJ 1151'
        if name=='GJ_324_A':
            name = 'GJ_324A'
        if name=='HD_68988':
            name = 'HD 68988'
        if name=='NLTT_51984':
            name = 'GJ_9751'
        self.name = name
        try:
            self.data = self.from_file(verbose=verbose)
        except Exception as e:
            print(e,'File does not exist!')
            if 'TIC' in name:
                print('Querying TIC for data')
                self.data = self.query_tic(name)
            else:
                print('Querying SIMBAD for data')
                self.data, self.warning = barycorrpy.utils.get_stellar_data(name)
            self.to_file(self.data)
        self.ra = self.data['ra']
        self.dec = self.data['dec']
        self.pmra = self.data['pmra']
        self.pmdec = self.data['pmdec']
        self.px = self.data['px']
        self.epoch = self.data['epoch']
        if self.data['rv'] is None:
            self.rv = 0.
        else:
            self.rv = self.data['rv']/1000.# if self.data['rv'] < 1e20 else 0.
        self.obsname = obsname


    def query_tic(self,ticname):
        """
        Query the TESS Input Catalog for data
        """
        name = ticname.replace('-',' ').replace('_',' ')
        df = Catalogs.query_object(name, radius=0.0003, catalog="TIC").to_pandas()[0:1]
        data = {}
        data['ra'] = df.ra.values[0]
        data['dec'] = df.dec.values[0]
        data['pmra'] = df.pmRA.values[0]
        data['pmdec'] = df.pmDEC.values[0]
        data['px'] = df.plx.values[0]
        data['epoch'] = 2451545.0
        data['rv'] = 0.
        return data

    def from_file(self,verbose=False):
        if verbose:
            print('Reading from file {}'.format(self.config_filename))
        #if os.path.exists(self.config_filename):
        config = configparser.ConfigParser()
        config.read(self.config_filename)
        data = dict(config.items('targetinfo'))
        for key in data.keys():
            data[key] = float(data[key])
        return data

    def to_file(self,data):
        print('Saving to file {}'.format(self.config_filename))
        config = configparser.ConfigParser()
        config.add_section('targetinfo')
        for key in data.keys():
            config.set('targetinfo',key,str(data[key]))
            print(key,data[key])
        with open(self.config_filename,'w') as f:
            config.write(f)
        print('Done')
        
    def calc_barycentric_velocity(self,jdtime,obs):
        """
        OUTPUT:
            BJD_TDB
            berv in km/s
        
        EXAMPLE:
            bjd, berv = bary.bjdbrv(H.jd_midpoint,T.ra,T.dec,obsname='McDonald Observatory',
                           pmra=T.pmra,pmdec=T.pmdec,rv=T.rv,parallax=T.px,epoch=T.epoch)
        """
        #bjd, berv = bary.bjdbrv(jdtime,self.ra,self.dec,obsname=self.obsname,
        #                           pmra=self.pmra,pmdec=self.pmdec,rv=self.rv,parallax=self.px,epoch=self.epoch)
        bjd, berv = bary.bjdbrv(jdtime, self.ra, self.dec, obsname=obs,
                                pmra=self.pmra, pmdec=self.pmdec, rv=self.rv, parallax=self.px, epoch=self.epoch)
        return bjd, berv/1000.
    
    def __repr__(self):
        return "{}, ra={:0.4f}, dec={:0.4f}, pmra={}, pmdec={}, rv={:0.4f}, px={:0.4f}, epoch={}".format(self.name,
                                            self.ra,self.dec,self.pmra,self.pmdec,self.rv,self.px,self.epoch)



import numpy as np
import matplotlib.pylab as plt
from PyAstronomy import pyasl

# Create data with a Gaussian absoprtion line
wvl = np.arange(4999., 5011., 0.04)
flux = np.zeros(len(wvl))

# The Gaussian
A = -0.05
s = 0.1
mu = 5004.1635788
flux += A/np.sqrt(2.*np.pi*s**2) * \
    np.exp(-(wvl-mu)**2/(2.*s**2))

# Apply the fast algorithm and ...
bfast = pyasl.fastRotBroad(wvl, flux, 0.81, 20)
# ... the slower one
bslow = pyasl.rotBroad(wvl, flux, 0.81, 20)

if __name__ == '__main__':
    plt.xlabel("Wvl [A]")
    plt.ylabel("Flux [au]")
    plt.title("Initial spectrum (black), fast (blue), slow (red, shifted)")
    plt.plot(wvl, flux, 'k.-')
    plt.plot(wvl, bfast, 'b.-')
    plt.plot(wvl, bslow, 'r.-')
    plt.show()



import sys
import os
import numpy as np
from pyneid.neid import Neid
from astropy.time import Time
import astropy.units as u
from astropy.table import Table, Column
from astropy.coordinates import SkyCoord
from tqdm import trange

DIRNAME = os.path.dirname(os.path.dirname(__file__))


def get_table_singlestar(starname, ra, dec, fmt, download=False, directory=None):
    print(starname)

    param = dict()
    param['datalevel'] = 'l2'
    param['object'] = starname
    param['radius'] = 1

    """
    other param options include:
        piname (e.g., "Suvrath Mahadevan")
        program (e.g., "2021B-2015")
        datetime (e.g., str(Time.now()-30*u.day)[:10]+" 00:00:00/")
    """

    if fmt == 'csv':
        outpath = starname.replace(' ', '_') + '_L2_q.csv'
    elif fmt == 'ipac':
        outpath = starname.replace(' ', '_') + '_L2_q.tbl'

    # Neid.query_criteria(param, \
    #                     cookiepath=f'{directory}neidadmincookie.txt', \
    #                     format=fmt, \
    #                     outpath=directory + outpath)
    Neid.query_position('l2', f'circle {ra} {dec} 0.5', cookiepath=f'{directory}neidadmincookie.txt', format=fmt,
                        outpath=directory + outpath)
    if fmt == 'ipac' and download == True:
        Neid.download(outpath,
                      'l2', \
                      'ipac', \
                      '.', \
                      cookiepath=f'{directory}neidadmincookie.txt')


if __name__ == '__main__':
    directory = '/Users/tehan/Documents/NEID_archive/'
    Neid.login(userid='than', password='QWAqwab-6766', cookiepath=f'{directory}neidadmincookie.txt',
               debugfile=f'{directory}archive.debug')
    combined = Table.read(f'{DIRNAME}/library/combined.csv', format='csv', )
    have_spectra = []
    for i in trange(len(combined)):  # len(combined)
        star = combined['Name'][i]
        ra = combined['RA'][i]
        dec = combined['Dec'][i]
        try:
            get_table_singlestar(star, ra, dec, 'csv', download=False, directory=directory)
            t = Table.read(f'{directory}{star.replace(" ", "_")}_L2_q.csv', format='csv')
            if len(t) > 0:
                have_spectra.append(star)
        except:
            continue
    print(have_spectra)
    result = Table([have_spectra], names=['Name'])
    result.write(f'{directory}have_spectra.csv', format='csv', overwrite=True)

    """
    if 'download' in sys.argv:
        fmt = 'ipac'
        download = True
    else:
        download = False
        fmt = 'csv'
    Example
    
    python3 -m pyneid_example_te user pw HD\ 143761
    
    then
    
    python3 -m pyneid_example_te user pw HD\ 143761 download
    """



import os
import glob2
import pandas as pd
import hpfspec
import utils, config
import neidspecmatch
from tqdm import trange

def run_hpfsm(targetname='TIC_39699837', orders=['55','101', '102', '103',]):
    filename = f'/home/tehan/Documents/hpf/spectra/Template_{targetname}_Fudge1.fits'
    utils.get_library()
    # List of stellar library fits files
    library_fitsfiles = config.LIBRARY_FITSFILES
    # Read in all files as a HPFSpecList object
    HLS =  hpfspec.HPFSpecList(filelist=library_fitsfiles)
    df_lib = pd.read_csv(config.PATH_LIBRARY_DB)

    outputdir = f'/home/tehan/Documents/hpf/{targetname}/'
    os.mkdir(outputdir)
    neidspecmatch.run_specmatch_for_orders(filename, targetname, outputdir, HLS=HLS, orders=orders)

    files = sorted(glob2.glob(f'{outputdir}*/*.pkl'))
    df_orders, df_orders_summary = neidspecmatch.summarize_values_from_orders(files, targetname)

    return df_orders, df_orders_summary


if __name__ == '__main__':
    filenames = glob2.glob('/home/tehan/Documents/hpf/spectra/*.fits')
    ### Gummi comment: orders with less tellurics: 3, 18, 19, 26
    for i in trange(len(filenames)):
        target = 'TIC_' + os.path.basename(filenames[i]).split('_')[2]
        try:
            run_hpfsm(targetname=target,
                             orders=['3', '4', '5', '6', '14', '15', '16', '17', '18', '19', '26'])
        except:
            continue



import numpy as np
import math
import pandas as pd

class PriorSet(object):
    def __init__(self, priors):
        self.priors = priors
        self.ndim  = len(priors)

        self.pmins = np.array([p.min() for p in self.priors])
        self.pmaxs = np.array([p.max() for p in self.priors])
        self.bounds= np.array([self.pmins,self.pmaxs]).T

        self.sbounds= np.array([[p.center-p.squeeze*0.5*p.width,
                                 p.center+p.squeeze*0.5*p.width] for p in self.priors])


    def generate_pv_population(self, npop):
        return np.array([[((p.random()[0]-p.center)*p.squeeze)+p.center for p in self.priors] for i in range(npop)])

    def c_log_prior(self, pv):
        return np.sum([p.log(v) for p,v in zip(self.priors, pv)])

    def c_prior(self, pv):
        return math.exp(self.c_log_prior(pv))

    @property
    def names(self):
        return [p.name for p in self.priors]
    
    # EXTENDING
    @property
    def descriptions(self):
        return [p.description for p in self.priors]
    
    @property
    def centers(self):
        return [p.center for p in self.priors]
    
    @property
    def labels(self):
        return [p.name for p in self.priors]
    
    @property
    def random(self):
        return [p.random()[0] for p in self.priors]
    
    @property
    def priortypes(self):
        return [p.priortype for p in self.priors]

    @property
    def args1(self):
        return [p.args1 for p in self.priors]

    @property
    def args2(self):
        return [p.args2 for p in self.priors]

    @property
    def ids(self):
        return [p.ID for p in self.priors]

    @property
    def fixed(self):
        return [p.fixed for p in self.priors]

    @property
    def df(self):
        return pd.DataFrame(list(zip(self.ids,self.args1,self.args2,self.labels,self.descriptions,self.priortypes,self.fixed)),
                columns=['prior','arg1','arg2','label','description','priortype','fixed'])
    
    @property
    def detrendparams(self):
        priortypes = self.priortypes

    def get_param_type_indices(self,paramtype="detrend"):
        assert (paramtype == "detrend" or paramtype=="model" or paramtype=="fixed" or paramtype=="gp")
        priortypes = self.priortypes
        df = pd.DataFrame(priortypes,columns=["priortypes"])
        return list(df[df["priortypes"] == paramtype].index)


class Prior(object):
    def __init__(self, a, b, name='', description='', unit='', squeeze=1.,priortype=""):
        self.a = float(a)
        self.b = float(b)
        self.center= 0.5*(a+b)
        self.width = b - a
        self.squeeze = squeeze

        self.name = name
        self.description = description
        self.unit = unit
        self.priortype = priortype

    def limits(self): return self.a, self.b 
    def min(self): return self.a
    def max(self): return self.b
   
 
class UniformPrior(Prior):
    def __init__(self, a, b, name='', description='', unit='', squeeze=1.,priortype="model"):
        self.args1 = a
        self.args2 = b
        self.ID = 'UP'
        super(UniformPrior, self).__init__(a,b,name,description,unit,squeeze,priortype)
        self._f = 1. / self.width
        self._lf = math.log(self._f)
        self.fixed = False

    def __call__(self, x, pv=None):
        if isinstance(x, np.ndarray):
            return np.where((self.a < x) & (x < self.b), self._f, 1e-80)
        else:
            return self._f if self.a < x < self.b else 1e-80

    def log(self, x, pv=None):
        if isinstance(x, np.ndarray):
            return np.where((self.a < x) & (x < self.b), self._lf, -1e18)
        else:
            return self._lf if self.a < x < self.b else -1e18

    def random(self, size=1):
        return np.random.uniform(self.a, self.b, size=size)


class JeffreysPrior(Prior):
    def __init__(self, a, b, name='', description='', unit='', squeeze=1.,priortype="model"):
        self.args1 = a
        self.args2 = b
        self.ID = 'JP'
        super(JeffreysPrior, self).__init__(a,b,name,description,unit,squeeze,priortype)
        self._f = math.log(b/a)
        self.fixed = False

    def __call__(self, x, pv=None):
        if isinstance(x, np.ndarray):
            return np.where((self.a < x) & (x < self.b), 1. / (x*self._f), 1e-80)
        else:
            return 1. / (x*self._f) if self.a < x < self.b else 1e-80

    def log(self, x, pv=None):
        if isinstance(x, np.ndarray):
            return np.where((self.a < x) & (x < self.b), np.log(1. / (x*self._f)), -1e18)
        else:
            return math.log(1. / (x*self._f)) if self.a < x < self.b else -1e18

    def random(self, size=1):
        return np.random.uniform(self.a, self.b, size=size)


class NormalPrior(Prior):
    def __init__(self, mean, std, name='', description='', unit='', lims=None, limsigma=5, squeeze=1,priortype="model"):
        self.args1 = mean
        self.args2 = std
        self.ID = 'NP'
        lims = lims or (mean-limsigma*std, mean+limsigma*std)
        super(NormalPrior, self).__init__(*lims, name=name, description=description, unit=unit,squeeze=squeeze,priortype=priortype)
        self.mean = float(mean)
        self.std = float(std)
        self._f1 = 1./ math.sqrt(2.*math.pi*std*std)
        self._lf1 = math.log(self._f1)
        self._f2 = 1./ (2.*std*std)
        self.fixed = False

    def __call__(self, x, pv=None):
        if isinstance(x, np.ndarray):
            return np.where((self.a < x) & (x < self.b),  self._f1 * np.exp(-(x-self.mean)**2 * self._f2), 1e-80)
        else:
            return self._f1 * exp(-(x-self.mean)**2 * self._f2) if self.a < x < self.b else 1e-80

    def log(self, x, pv=None):
        if isinstance(x, np.ndarray):
            return np.where((self.a < x) & (x < self.b),  self._lf1 - (x-self.mean)**2 * self._f2, -1e18)
        else:
            return self._lf1 -(x-self.mean)**2*self._f2 if self.a < x < self.b else -1e18

    def random(self, size=1):
        return np.random.uniform(self.a, self.b, size=size) #normal(self.mean, self.std, size)
    
class FixedPrior(Prior):
    def __init__(self, value, name='', description='', unit='', lims=None, squeeze=1,priortype="model"):
        self.args1 = value
        self.args2 = value
        self.ID = 'FP'
        lims = value, value
        super(FixedPrior, self).__init__(*lims, name=name, description=description, unit=unit,squeeze=squeeze,priortype=priortype)
        self.mean = float(value)
        self.fixed = True

    def __call__(self, x, pv=None):
        if isinstance(x, np.ndarray):
            return self.mean*np.ones(len(x))
        else:
            return self.mean

    def log(self, x, pv=None):
        return np.zeros_like(x) if isinstance(x, np.ndarray) else 0.

    #def log(self, x, pv=None):
    #    if isinstance(x, np.ndarray):
    #        return np.where((self.a < x) & (x < self.b),  self._lf1 - (x-self.mean)**2 * self._f2, -1e18)
    #    else:
    #        return self._lf1 -(x-self.mean)**2*self._f2 if self.a < x < self.b else -1e18

    #def random(self, size=1):
    #    return np.random.uniform(self.a, self.b, size=size) #normal(self.mean, self.std, size)

class DummyPrior(Prior):
    def __init__(self, a, b, name='', description='', unit='',priortype="model"):
        super(DummyPrior, self).__init__(a, b, name=name, description=description, unit=unit,priortype="model")

    def __call__(self, x, pv=None):
        return np.ones_like(x) if isinstance(x, np.ndarray) else 1.

    def log(self, x, pv=None):
        return np.zeros_like(x) if isinstance(x, np.ndarray) else 0.

    def random(self, size=1):
        return np.random.uniform(self.a, self.b, size=size)


UP = UniformPrior
JP = JeffreysPrior
GP = NormalPrior
NP = NormalPrior 
DP = DummyPrior
FP = FixedPrior



import matplotlib.pyplot as plt
import glob2
from tqdm import trange
import os
from astropy.io import ascii
import numpy as np
from astropy.io import fits


def plot_orders():
    filenames = glob2.glob('/home/tehan/Documents/hpf/spectra/*.fits')
    star_num = 0
    vsinis = np.zeros((11, len(filenames)))
    teffs = np.zeros((11, len(filenames)))
    logg = np.zeros((11, len(filenames)))
    feh = np.zeros((11, len(filenames)))
    star_id = []
    for i in trange(len(filenames)):
        target = 'TIC_' + os.path.basename(filenames[i]).split('_')[2]
        try:
            data = ascii.read(f'/home/tehan/Documents/hpf/{target}/{target}_overview.csv', format='csv')
            star_num += 1
            star_id.append(target)
        except FileNotFoundError:
            continue
        teffs[:, i] = data['teff']
        vsinis[:, i] = data['vsini']
        logg[:, i] = data['logg']
        feh[:, i] = data['feh']
        # median_teff = np.median(data['teff'])
        # median_vsini = np.median(data['vsini'])
        # colors = ['C0', 'C1', 'C2', 'C3', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9', 'k']
        # for j in range(len(data)):
        #     order = data['filenames'][j].split('/')[-2].split('_')[-1]
        #     if star_num == 1:
        #         plt.scatter(star_num, np.log10(data['vsini'][j] / median_vsini), color=colors[j], marker='x',
        #                     linewidths=3, s=100, label=order)
        #     else:
        #         plt.scatter(star_num, np.log10(data['vsini'][j] / median_vsini), color=colors[j], marker='x',
        #                     linewidths=3, s=100)
    idx = np.argwhere(np.all(vsinis[..., :] == 0, axis=0))
    vsinis = np.delete(vsinis, idx, axis=1)
    teffs = np.delete(teffs, idx, axis=1)
    logg = np.delete(logg, idx, axis=1)
    feh = np.delete(feh, idx, axis=1)

    sort_id = np.argsort(np.median(teffs, axis=0))
    star_id = np.array(star_id)[sort_id]
    vsinis = vsinis[:, sort_id]
    teffs = teffs[:, sort_id]
    logg = logg[:, sort_id]
    feh = feh[:, sort_id]
    colors = ['C0', 'C1', 'C2', 'C3', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9', 'k']
    orders = ['14', '15', '16', '17', '18', '19', '26', '3', '4', '5', '6']

    fig = plt.figure(constrained_layout=False, figsize=(8, 10))
    gs = fig.add_gridspec(4, 1)
    gs.update(wspace=0.2, hspace=0.05)
    ax1 = fig.add_subplot(gs[0, 0])
    ax2 = fig.add_subplot(gs[1, 0])
    ax3 = fig.add_subplot(gs[2, 0])
    ax4 = fig.add_subplot(gs[3, 0])

    for j in range(11):
        ax1.scatter(np.arange(len(vsinis[0])), np.log10(vsinis[j] / np.median(vsinis, axis=0)), color=colors[j],
                    marker='x',
                    linewidths=3, s=100, label=orders[j])
        ax1.plot(np.arange(len(vsinis[0])), np.log10(vsinis[j] / np.median(vsinis, axis=0)), c=colors[j], marker='')

        ax2.scatter(np.arange(len(vsinis[0])), (teffs[j] / np.median(teffs, axis=0)), color=colors[j], marker='x',
                    linewidths=3, s=100, label=orders[j])
        ax2.plot(np.arange(len(vsinis[0])), (teffs[j] / np.median(teffs, axis=0)), c=colors[j], marker='')

        ax3.scatter(np.arange(len(vsinis[0])), (logg[j] / np.median(logg, axis=0)), color=colors[j], marker='x',
                    linewidths=3, s=100, label=orders[j])
        ax3.plot(np.arange(len(vsinis[0])), (logg[j] / np.median(logg, axis=0)), c=colors[j], marker='')

        ax4.scatter(np.arange(len(vsinis[0])), (feh[j] / np.median(feh, axis=0)), color=colors[j], marker='x',
                    linewidths=3, s=100, label=orders[j])
        ax4.plot(np.arange(len(vsinis[0])), (feh[j] / np.median(feh, axis=0)), c=colors[j], marker='')

    # ax2.errorbar(np.arange(len(vsinis[0])), np.median(vsinis, axis=0), np.std(vsinis, axis=0), capsize=3, ls='')
    # ax3.errorbar(np.arange(len(vsinis[0])), np.median(teffs, axis=0), np.std(teffs, axis=0), capsize=3, ls='')
    for k in range(18):
        ax4.text(np.arange(len(vsinis[0]))[k] - 0.3, -55, star_id[k], verticalalignment='top', rotation=-90,
                 fontweight='semibold')
    # ax3.set_xlabel('Star No.')
    ax1.set_ylabel('log(vsini)')
    ax2.set_ylabel('teff')
    ax3.set_ylabel('logg')
    ax4.set_ylabel('feh')
    ax1.set_xticklabels([])
    ax2.set_xticklabels([])
    ax3.set_xticklabels([])
    ax4.set_xticklabels([])
    ax1.tick_params(axis='x', bottom=False)
    ax2.tick_params(axis='x', bottom=False)
    ax3.tick_params(axis='x', bottom=False)
    ax4.tick_params(axis='x', bottom=False)

    plt.gcf().subplots_adjust(bottom=0.15, top=0.95)
    ax4.legend(ncol=3)
    plt.savefig('/home/tehan/Documents/hpf/hpf_orders.pdf', dpi=300)
    plt.show()


def plot_lc_each_sector(local_directory=None):
    files = glob2.glob(f'{local_directory}*.fits')
    os.makedirs(f'{local_directory}plots/', exist_ok=True)
    for i in range(len(files)):
        with fits.open(files[i], mode='denywrite') as hdul:
            plt.figure(constrained_layout=False, figsize=(8, 4))
            plt.plot(hdul[1].data['time'], hdul[1].data['cal_psf_flux'], '.', label='cal_psf')
            plt.plot(hdul[1].data['time'], hdul[1].data['cal_aper_flux'], '.', label='cal_aper')
            plt.title(f'TIC_{hdul[0].header["TICID"]}')
            plt.legend()
            plt.savefig(f'{local_directory}plots/TIC_{hdul[0].header["TICID"]}.png', dpi=300)

def plot_lc(local_directory=None, catalog=None):
    targets = ascii.read(catalog, format='csv')
    os.makedirs(f'{local_directory}plots/', exist_ok=True)
    for i in trange(len(targets)):
        files = glob2.glob(f'{local_directory}*{targets["GaiaDR3"][i]}*.fits')
        if len(files) == 0:
            continue
        else:
            fig = plt.figure(figsize=(13, 5))
            for j in range(len(files)):
                with fits.open(files[j], mode='denywrite') as hdul:
                    q = list(hdul[1].data['TESS_flags'] == 0) and list(hdul[1].data['TGLC_flags'] == 0)
                    period = targets['period'][np.where(targets['TIC'] == int(hdul[0].header['TICID']))]
                    plt.plot(hdul[1].data['time'] % period, hdul[1].data['cal_aper_flux'], '.', c='silver', ms=1)
                    plt.plot(hdul[1].data['time'][q] % period, hdul[1].data['cal_aper_flux'][q], '.', c="C0", ms=1)
                    title = f'TIC_{hdul[0].header["TICID"]} with {len(files)} sector(s) of data'
            if targets['TIC'][i] == 236785891:
                plt.ylim(0.6, 1.4)
            plt.title(title)
            plt.xlabel('Phase (days)')
            plt.ylabel('Normalized flux')
            plt.savefig(f'{local_directory}/plots/{title}.png',
                        dpi=300)
            plt.close(fig)


import numpy as np
import matplotlib.pyplot as plt


def plot_vsini_limit(file='/Users/tehan/PycharmProjects/neidspecmatch/all_vsinis_he.npy'):
    all_vsinis = np.load(file, allow_pickle=True)
    injected_vsini = np.linspace(10, 0.01, 20)[:len(all_vsinis)]

    fig, axs = plt.subplots(2, 1, figsize=(6, 8), sharex=True, gridspec_kw={'height_ratios': [2, 1]})

    # Main plot: Injected vs. Recovered vsini
    axs[0].plot(injected_vsini, all_vsinis[:, 0], marker='.', label='i55')
    axs[0].plot(injected_vsini, all_vsinis[:, 1], marker='.', label='i101')
    axs[0].plot(injected_vsini, all_vsinis[:, 2], marker='.', label='i102')
    axs[0].plot(injected_vsini, all_vsinis[:, 3], marker='.', label='i103')
    axs[0].plot([0, 10], [0, 10], 'k')
    axs[0].set_ylabel('Recovered vsini')
    axs[0].legend()

    # Residual plot: Difference between recovered and injected
    for i in range(4):
        axs[1].plot(injected_vsini, all_vsinis[:, i] - injected_vsini, marker='.', label=f'Residual {i}')

    axs[1].axhline(0, color='k', linestyle='--')
    axs[1].set_xlabel('Injected vsini')
    axs[1].set_ylabel('Residual')

    plt.savefig('HE_vsini_recovery.png')
    plt.show()


if __name__ == '__main__':
    # plot_lc(local_directory = '/home/tehan/Documents/GEMS/lc/',
    #         catalog = '/home/tehan/Documents/GEMS/GEMS.csv')
    plot_vsini_limit()


from numpy import square, log, ndarray, pi
from math import log as mlog
LOG_TWO_PI = log(2*pi)

def ll_normal_es_py(o,m,e):
    """Normal log likelihood for scalar average standard deviation."""
    npt = o.size
    return -npt*mlog(e) -0.5*npt*LOG_TWO_PI - 0.5*square(o-m).sum()/e**2

def ll_normal_ev_py(o,m,e):
    """Normal log likelihood for varying e"""
    npt = o.size
    return -log(e).sum() -0.5*npt*LOG_TWO_PI - 0.5*square((o-m)/e).sum()




import barycorrpy
from astropy.time import Time
from astropy import time, coordinates as coord, units as u

def bjdbrv(jd_utc, ra=None, dec=None, obsname=None, lat=0., lon=0., elevation=None,
        pmra=0., pmdec=0., parallax=0., rv=0., zmeas=0.,
        epoch=2451545.0, tbase=0., leap_update=True,**kwargs):
    """
    Wrapper to barycorrpy.py and utc2bjd. Computes the barycentric
    velocity correction and julian date in one call.
    Keyword obsname refers to observatory.pro in the IDL Astronomy User Library

    See also: http://astroutils.astronomy.ohio-state.edu/exofast/barycorr.html

    :param jd_utc: Julian date (UTC)
    :param ra: RA (J2000) [deg]
    :param dec: Dec (J2000) [deg]
    :param obsname: Observatory name (overrides coordinates if set)
    :param lat: Observatory latitude  [deg]
    :param lon: Observatory longitude (E) [+/-360 deg]
    :param elevation: Observatory elevation [m]
    :param pmra: Proper motion (RA*cos(Dec)) [mas/yr]
    :param pmdec: Proper motion (Dec) [mas/yr]
    :param parallax: Parallax [mas]
    :param rv: Radial velocity (within 100 km/s) [m/s]
    :param zmeas: Measured redshift
    :param epoch: Epoch (default 2448348.56250, J2000)
    :param tbase: Baseline subtracted from times (default 0.0)
    :return: Barycentric correction for zmeas

    Example:
    --------
    >>> from brv_we14py import bjdbrv
    >>> print bjdbrv(2457395.24563, 4.585590721,  44.02195596, 'ca')
    (2457395.247062386, -23684.54364462639)

    """
    if obsname=="Kitt Peak National Observatory": # Same as used in SERVAL
        lat = 31.9580
        lon = -111.6006
        elevation = 2090.
    # Barycentric Julian Date
    # adapted from http://docs.astropy.org/en/stable/time/#barycentric-and-heliocentric-light-travel-time-corrections
    targ = coord.SkyCoord(ra, dec, unit=(u.deg, u.deg), frame='icrs')
    loc = coord.EarthLocation.from_geodetic(lon, lat, height=elevation)
    #times = time.Time(jd_utc, format='jd', scale='utc', location=loc)
    #ltt_bary = times.light_travel_time(targ)
    JDUTC = Time(jd_utc, format='jd', scale='utc')
    ltt_bary = JDUTC.light_travel_time(targ, location=loc)
    bjd = JDUTC.tdb + ltt_bary
   
    # we should be JDUTC
    if leap_update is False:
        print('WARNING: LEAP UPDATE=FALSE')
    brv, warning, status = barycorrpy.get_BC_vel(JDUTC, ra=ra, dec=dec, epoch=epoch, pmra=pmra,
                    pmdec=pmdec, px=parallax, lat=lat, longi=lon, alt=elevation,leap_update=leap_update,**kwargs)
    if len(brv) > 1:
        return bjd.value, brv
    else:
       return bjd.value, brv[0]



import numpy as np
import os, sys
import glob
from astropy.io import fits
from scipy.interpolate import interp1d
from astropy.constants import c
ProjectDirectory = r"/storage/home/szk381/cfb12_link/default/SpectralAnalysis"
ProcessedFITSDirectory = os.path.join(ProjectDirectory, 'ProcessedFITS')

try:
    pwd = os.path.dirname(__file__)
except:
    pwd = os.path.join(ProjectDirectory, 'SpectraWorking', 'src')
sys.path.append(pwd)

import target


def CombineNEIDSpectra(ObjectName, TemplateFilePath, NormalizationOrder=5, TelluricFudgeFactor=1):
    """
    Combine spectra to create template for NEID

    INPUTS:
        ObjectName: ObjectName to query target parameters for (required for barycentric correction)
        TemplateFilePath: File Path to the template files that will be combined
        TemplateFileList: List of file names to be combined. It is assumed that all files are for the same target
        NormalizationOrder: Normalize all the spectra to this order using their 95% percentile
        TelluricFudgeFactor: Telluric fudge factor to increase the variance where tel mask is true.

    OUTPUTS:
        HDU object corresponding to template file. This is the HDU object for the first template file, with the Science wl, flux and variance replaced by those for the template.
            The template wavelength used is in vacuum in the stellar rest frame.

    """

    NumOrders = 94
    NumPixels = 9216
    TemplateFileList = glob.glob(os.path.join(TemplateFilePath, 'neid*.fits'))
    SciFluxMaster = np.zeros((len(TemplateFileList), NumOrders, NumPixels))
    SciFluxVarianceMaster = np.zeros((len(TemplateFileList), NumOrders, NumPixels))
    NormalizationFactor = np.zeros((len(TemplateFileList)))
    Normalization_1st = None
    for i, t in enumerate(TemplateFileList):
        print("Processing and weighting frame = " + t)

        HDU = fits.open(t)
        # Header = HDU[0].header
        SciWavelength = HDU[7].data[10:104]
        SciFlux = HDU[1].data[10:104]
        SciVariance = HDU[4].data[10:104]

        # FluxWeightedJD = np.array([Header['JD_FW{}'.format(m)] for m in range(28)])
        # IntegrationTime = Header['EXPLNDR']

        # Replace the dummy grid below with the real telluric mask which should be true where tellurics are present.
        TelluricModel = np.zeros(np.shape(SciFlux)).astype(bool)
        TelMask = TelluricModel < 0.9 * np.nanmedian(TelluricModel, axis=1)[:, None]
        SciVariance[TelMask] *= TelluricFudgeFactor  # Inflate variance in Sci flux for telluric line regions

        T = target.Target(ObjectName)
        T.bjd, T.berv = T.calc_barycentric_velocity(HDU[0].header['OBSJD'], 'Kitt Peak National Observatory')
        zbary = T.berv * 1000 / c.value
        zbulk = T.rv * 1000 / c.value
        SciWlStellar = np.zeros(np.shape(SciWavelength))

        # Multiply for zbary because observer is moving
        # Divide by zbulk because emitter is moving
        for o in range(NumOrders):
            SciWlStellar[o] = (SciWavelength[o]* (1 + zbary) / (1 + zbulk))

        if i == 0:
            WavelengthReference = np.copy(SciWlStellar)
            # TelluricMaskGrid = TelluricModel
            # ReferenceBCVels = T.berv
            RefHDU = HDU

        # Need to interpolate spectra on to the reference grid
        SciFluxInterp = np.zeros((np.shape(SciFlux)))
        SciVarianceInterp = np.zeros((np.shape(SciFlux)))

        # Use just one normalization for each epoch. NOT separate for each order.
        NormalizationRaw = np.nanpercentile(SciFlux[NormalizationOrder], q=95)

        if i ==0:
            NormalizationFactor[i] = 1
            Normalization_1st = NormalizationRaw
        else:
            NormalizationFactor[i] = NormalizationRaw / Normalization_1st
        print(NormalizationFactor)
        for Order in range(NumOrders):
            nanmask = np.isnan(SciFlux[Order])
            SciFluxInterp[Order] = interp1d(SciWlStellar[Order][~nanmask], SciFlux[Order][~nanmask], kind='linear',
                                            fill_value='extrapolate')(WavelengthReference[Order])
            SciVarianceInterp[Order] = interp1d(SciWlStellar[Order][~nanmask], SciVariance[Order][~nanmask],
                                                kind='linear', fill_value='extrapolate')(WavelengthReference[Order])

        SciFluxMaster[i] = SciFluxInterp / NormalizationFactor[i]
        SciFluxVarianceMaster[i]= SciVarianceInterp / (NormalizationFactor[i] ** 2)
    SciFluxAveraged = np.average(SciFluxMaster, axis=0, weights=1 / SciFluxVarianceMaster, returned=True)
    WeightedSciFlux = SciFluxAveraged[0]
    WeightedSciVariance = 1 / SciFluxAveraged[1]
    RefHDU[1].data[10:104] = WeightedSciFlux
    RefHDU[4].data[10:104] = WeightedSciVariance
    RefHDU[7].data[10:104] = WavelengthReference  # Wavelength in vacuum in stellar rest frame
    # RefHDU[0].header['FilesAveraged'] = TemplateFileList
    RefHDU.writeto(f'{TemplateFilePath}/{ObjectName}_{len(TemplateFileList)}_SpectraAveraged.fits', overwrite=True)
    return RefHDU

if __name__ == '__main__':
    hdul = CombineNEIDSpectra('TIC 437039407', '/Users/tehan/Documents/NEID_archive/14020_Spectra')
    print(hdul)


import neidspec
import numpy as np
import pandas as pd
import astropy.io
import pickle
import os
from neidspec import stats_help
import pyde
import pyde.de
import emcee
import scipy.optimize
from neidspec import rotbroad_help
from neidspec import utils
import matplotlib.pyplot as plt
import astropy.modeling
from neidspecmatch.priors import PriorSet, UP
from neidspecmatch.likelihood import ll_normal_ev_py
import neidspecmatch.config as config
from matplotlib.gridspec import GridSpec
from matplotlib import rcParams

rcParams["savefig.dpi"] = 100
rcParams['mathtext.fontset'] = 'stix'
rcParams['font.family'] = 'STIXGeneral'
rcParams['font.weight'] = "normal"
rcParams["axes.formatter.useoffset"] = False
rcParams['xtick.direction'] = 'in'
rcParams['ytick.direction'] = 'in'


def get_data_ready(H1, Hrefs, w, v, polyvals=None, vsinis=None, order=101, plot=False, deblazed=False):
    """
    Get data ready for MCMC
    
    INPUT:
        H1 - target spectrum (HPFSpectrum object)
        Hrefs - reference spectra (HPFSpectraList object)
        w - wavelength grid to interpolate on (array)
        v - velocities in km/s to use for absolute RV consideration (array)
        polyvals - polynomial coefficients (array)
        vsinis - vsini values km/s (array)
        plot - (boolean)

    OUTPUT:
        f1 - spectrum
        e1 - spectrum flux error
        ffrefs - spectra  (array)
        eerefs - spectra flux errors (array)
       
    EXAMPLE:
        "Target={}, rv={:0.3f}km/s, rvmed={:0.3f}km/s".format(H1.target.name,H1.rv,np.median(rabs))
        files = sorted(glob.glob('20200209_ad_leos/AD_Leo/*/*.pkl'))
        summarize_values_from_orders(files,'AD_Leo')
    """
    if not deblazed:
        H1.deblaze()
    _, rabs = H1.rvabs_for_orders(v, orders=[55,56,91], plot=plot)
    # _, rabs = H1.rvabs_for_orders(v, orders=[91,92,93], plot=plot)
    print(rabs)
    H1.redshift(rv=np.median(rabs))
    f1, e1 = H1.resample_order(w, plot=True, order=order, deblazed=deblazed)
    print("Target={}, rv={:0.3f}km/s, rvmed={:0.3f}km/s".format(H1.target.name, H1.rv, np.median(rabs)))

    ffrefs = []
    eerefs = []
    for i, H in enumerate(Hrefs.splist):
        H.deblaze()
        _, rabs = H.rvabs_for_orders(v, orders=[55,56,91], plot=plot)
        H.redshift(rv=np.median(rabs))
        if polyvals is None and vsinis is None:
            _f, _e = H.resample_order(w, order=order)
        else:
            _f, _e = H.resample_order(w, p=polyvals[i], vsini=vsinis[i], order=order)
        ffrefs.append(_f)
        eerefs.append(_e)
        print("Target={}, rv={:0.3f}km/s, rvmed={:0.3f}km/s".format(H1.target.name, H1.rv, np.median(rabs)))
        print('HR mode has an LSF width of the instrument resolution of 2.8 km/s;')
        print('HE mode has an LSF width of the instrument resolution of 4.5 km/s.')
    # plt.show()
    # plt.plot(w, f1)
    # plt.savefig('/home/tehan/Downloads/target.png')
    return f1, e1, ffrefs, eerefs


class LPFunctionLinComb(object):
    """
    Log Likelihood function for optimizing a combination of 5 top spectra
    """

    def __init__(self, w, f1, e1, f2s, e2s):
        self.w = w
        self.data_target = {'f': f1,
                            'e': e1}
        self.data_refs = {'f': f2s,
                          'e': e2s}
        self.num_refs = len(self.data_refs['f'])

        self.priors = [UP(0., 1., 'c1', '$c1$', priortype="model"),
                       UP(0., 1., 'c2', '$c2$', priortype="model"),
                       UP(0., 1., 'c3', '$c3$', priortype="model"),
                       UP(0., 1., 'c4', '$c4$', priortype="model")]
        self.ps = PriorSet(self.priors)

    def get_pv_all(self, pv):
        pv_all = list(pv) + [1. - np.sum(pv)]
        return pv_all

    def compute_model(self, pv):
        pv_all = self.get_pv_all(pv)
        return np.sum([self.data_refs['f'][i] * pv_all[i] for i in range(self.num_refs)], axis=0)

    def __call__(self, pv):
        if 1. - np.sum(pv) < 0. or 1. - np.sum(pv) > 1.:
            return -np.inf
        if any(pv < self.ps.pmins) or any(pv > self.ps.pmaxs):
            return -np.inf
        flux_model = self.compute_model(pv)
        flux_target = self.data_target['f']
        error_target = self.data_target['e']
        log_of_priors = self.ps.c_log_prior(pv)
        log_of_model = ll_normal_ev_py(flux_target, flux_model, error_target)
        log_ln = log_of_priors + log_of_model
        # log_constraint = (np.sum(pv)-1.)/(2.*self.sigma)
        # print('lnPrior = {} lnModel = {} lnExtra = {}'.format(log_of_priors,log_of_model,log_constraint))
        return log_ln


class FitLinCombSpec(object):
    """
    A class to fit 5 lin-comb spectra together. Note: look at LPFunctionLinComb
    """

    def __init__(self, LPFunctionLinComb, teffs=[], fehs=[], loggs=[], vsinis=[],
                 teff_known=None, tefferr_known=None,
                 feh_known=None, feherr_known=None,
                 logg_known=None, loggerr_known=None, targetname='', calibrate_feh=True):
        self.lpf = LPFunctionLinComb
        self.teffs = teffs
        self.fehs = fehs
        self.loggs = loggs
        self.vsinis = vsinis
        self.teff_known = teff_known
        self.tefferr_known = tefferr_known
        self.feh_known = feh_known
        self.feherr_known = feherr_known
        self.logg_known = logg_known
        self.loggerr_known = loggerr_known
        self.targetname = targetname
        self.calibrate_feh = calibrate_feh

    def calculate_stellar_parameters(self, weights):
        if self.teffs != []:
            self.teff = weighted_value(self.teffs, weights)
        else:
            self.teff = np.nan
        if self.fehs != []:
            print(self.fehs)
            print(weights)
            self.feh = weighted_value(self.fehs, weights)
            if self.calibrate_feh:
                print('Calibrating feh: {:0.3f} -> {:0.3f}'.format(self.feh, detrend_feh(self.feh)))
                self.feh = detrend_feh(self.feh)
        else:
            self.feh = np.nan
        if self.loggs != []:
            self.logg = weighted_value(self.loggs, weights)
        else:
            self.logg = np.nan
        if self.vsinis != []:
            self.vsini = weighted_value(self.vsinis, weights)
        print('Stellar parameters:')
        print('Teff [K]:', self.teff)
        print('Fe/H [dex]:', self.feh)
        print('logg [dex]:', self.logg)
        print('vsini [km/s]:', self.vsini)

    def plot_model(self, pv):
        pv_all = self.lpf.get_pv_all(pv)
        fig, (ax, bx) = plt.subplots(nrows=2, dpi=200, sharex=True, gridspec_kw={'height_ratios': [5, 2]})
        ax.plot(self.lpf.w, self.lpf.data_target['f'], color='black', label='Target', lw=1)
        ff = self.lpf.compute_model(pv)

        ax.plot(self.lpf.w, ff, color='crimson', label='Composite', alpha=0.5, lw=1)
        ax.legend(fontsize=10)
        bx.plot(self.lpf.w, self.lpf.data_target['f'] - ff, lw=1)
        bx.set_xlabel('Wavelength [A]', fontsize=12)
        bx.set_ylabel('Residual', fontsize=12)

        self.calculate_stellar_parameters(pv_all)

        title = 'log_ln={}\n'.format(self.lpf(pv))
        title += ' '.join(['c_{}={:0.5f}'.format(i, pv_all[i]) for i in range(5)])
        title += '\nTeff={:0.3f}, Fe/H={:0.3f}, logg={:0.3f}'.format(self.teff, self.feh, self.logg)
        if self.teff_known is not None and self.feh_known is not None and self.logg_known is not None:
            title += '\nTeff_0={:0.3f}+-{:0.1f}K, Fe/H_0={:0.3f}+-{:0.3f}, logg_0={:0.3f}+-{:0.3f}'.format(
                self.teff_known, self.tefferr_known, self.feh_known, self.feherr_known, self.logg_known,
                self.loggerr_known)
        ax.set_title(title, fontsize=10)
        for xx in (ax, bx):
            utils.ax_apply_settings(xx, ticksize=10)
        fig.subplots_adjust(hspace=0.05)

    def plot_model_with_components(self, pv, fig=None, ax=None, names=None, savename='compositeComparison.pdf',
                                   title='', scaleres=1., mode='HR'):
        """
        INPUT:
            scaleres - amount to scale residuals from composite spectrum (default 1)
        Make a plot of the 5 best stars and compare to targets spectrum.
        Compare composite spectrum to target spectrum and plot residuals.
        """
        w = self.lpf.w
        pv_all = self.lpf.get_pv_all(pv)
        self.calculate_stellar_parameters(pv_all)
        if fig is None and ax is None:
            fig, ax = plt.subplots(dpi=200)
        ax.plot(w, self.lpf.data_target['f'], color='black', label='Target', lw=1)
        ff = self.lpf.compute_model(pv)
        ax.plot(w, ff, color='crimson', label='Composite', alpha=0.5, lw=1.5)

        for i in range(5):
            ax.plot(w, self.lpf.data_refs['f'][i] + (5.0 - i), lw=1, color='black')
            if names is not None:
                label = '{}, $c_{}$={:0.5f}'.format(names[i].replace('_', ' '), i + 1, self.lpf.get_pv_all(pv)[i])
                label += ', Teff={:0.0f}K'.format(self.teffs[i])
                label += ', [Fe/H]={:0.2f}'.format(self.fehs[i])
            else:
                label = '$c_{}$={:0.5f}'.format(i + 1, pv_all[i])
                label += ', Teff={:0.0f}K'.format(self.teffs[i])
                label += ', [Fe/H]={:0.2f}'.format(self.fehs[i])
            ax.text(w[0], (6.12 - i), label, color='black', fontsize=8)

        ax.text(w[0], 1.15, 'Target Spectrum (Black), Composite Spectrum (Red)', fontsize=8)
        ax.text(w[0], 0.15, 'Residual: Target - Composite (Scale: {:0.0f}x)'.format(scaleres), fontsize=8)
        if mode == 'HR':
            if self.vsini < 2.8:
                title += 'Target={}, Teff={:0.3f}, Fe/H={:0.3f}, logg={:0.3f}, vsini<{:0.3f}km/s'.format(
                    self.targetname,self.teff, self.feh,self.logg, 2.8)
            else:
                title += 'Target={}, Teff={:0.3f}, Fe/H={:0.3f}, logg={:0.3f}, vsini={:0.3f}km/s'.format(
                    self.targetname,self.teff, self.feh,self.logg, self.vsini)
        elif mode == 'HE':
            if self.vsini < 4.5:
                title += 'Target={}, Teff={:0.3f}, Fe/H={:0.3f}, logg={:0.3f}, vsini<{:0.3f}km/s'.format(
                    self.targetname,self.teff, self.feh,self.logg, 4.5)
            else:
                title += 'Target={}, Teff={:0.3f}, Fe/H={:0.3f}, logg={:0.3f}, vsini={:0.3f}km/s'.format(
                    self.targetname,self.teff, self.feh,self.logg, self.vsini)

        ax.set_title(title, fontsize=10)

        ax.plot(w, (self.lpf.data_target['f'] - ff) * scaleres, color='black', lw=1)
        ax.set_xlabel('Wavelength [A]', fontsize=12, labelpad=2)
        ax.set_ylabel('Flux (+offset)', fontsize=12, labelpad=2)
        ax.set_ylim(-1,7)
        # ax.set_xlim(8645, 8650)
        print('___' + self.targetname)
        if self.targetname == 'TIC 268804174':
            ax.set_xlim(8645,8650)
        elif self.targetname == 'TIC 365159477':
            ax.set_xlim(8605,8615)
        elif self.targetname == 'TIC 248027247':
            ax.set_xlim(8650,8655)
        # ax.set_title('log_ln={}, c={}'.format(self.lpf(pv),str(self.lpf.get_pv_all(pv))),fontsize=10)
        utils.ax_apply_settings(ax, ticksize=10)
        fig.tight_layout()
        fig.savefig(savename, dpi=200)
        print('Saved to {}'.format(savename))

    def plot_model_with_component_save(self, pv, fig=None, ax=None, names=None, savename='compositeComparison.pdf',
                                   title='', scaleres=1., mode='HR', datafile='plot_data.pkl'):
        """
        INPUT:
            scaleres - amount to scale residuals from composite spectrum (default 1)
        Make a plot of the 5 best stars and compare to targets spectrum.
        Compare composite spectrum to target spectrum and plot residuals.
        """
        w = self.lpf.w
        pv_all = self.lpf.get_pv_all(pv)
        self.calculate_stellar_parameters(pv_all)
        target_spectrum = self.lpf.data_target['f']
        composite_spectrum = self.lpf.compute_model(pv)
        ref_spectra = [self.lpf.data_refs['f'][i] for i in range(5)]
        labels = []
        for i in range(5):
            if names is not None:
                label = '{}, $c_{}$={:0.5f}'.format(names[i].replace('_', ' '), i + 1, pv_all[i])
                label += ', Teff={:0.0f}K'.format(self.teffs[i])
                label += ', [Fe/H]={:0.2f}'.format(self.fehs[i])
            else:
                label = '$c_{}$={:0.5f}'.format(i + 1, pv_all[i])
                label += ', Teff={:0.0f}K'.format(self.teffs[i])
                label += ', [Fe/H]={:0.2f}'.format(self.fehs[i])
            labels.append(label)

        # Save data to a pickle file
        plot_data = {
            'wavelength': w,
            'target_spectrum': target_spectrum,
            'composite_spectrum': composite_spectrum,
            'ref_spectra': ref_spectra,
            'labels': labels,
            'title': title,
            'scaleres': scaleres,
            'mode': mode,
            'vsini': self.vsini,
            'targetname': self.targetname,
            'teff': self.teff,
            'feh': self.feh,
            'logg': self.logg
        }
        with open(datafile, 'wb') as f:
            pickle.dump(plot_data, f)
        print(f'Saved plot data to {datafile}')

        # ... (rest of your plotting code here)

    def minimize_PyDE(self, npop=100, de_iter=200, mc_iter=1000, mcmc=True, threads=8, maximize=True, plot_priors=True,
                      sample_ball=False):
        """
        Minimize using the PyDE
        
        NOTES:
        https://github.com/hpparvi/PyDE
        """
        centers = np.array(self.lpf.ps.centers)
        print("Running PyDE Optimizer")
        self.de = pyde.de.DiffEvol(self.lpf, self.lpf.ps.bounds, npop,
                                   maximize=maximize)  # we want to maximize the likelihood
        self.min_pv, self.min_pv_lnval = self.de.optimize(ngen=de_iter)
        print("Optimized using PyDE")
        print("Final parameters:")
        self.print_param_diagnostics(self.min_pv)
        # self.lpf.ps.plot_all(figsize=(6,4),pv=self.min_pv)
        print("LogLn value:", self.min_pv_lnval)
        print("Log priors", self.lpf.ps.c_log_prior(self.min_pv))
        if mcmc:
            print("Running MCMC")
            self.sampler = emcee.EnsembleSampler(npop, self.lpf.ps.ndim, self.lpf, threads=threads)
            print("MCMC iterations=", mc_iter)
            for i, c in enumerate(self.sampler.sample(self.de.population, iterations=mc_iter)):
                print(i, end=" ")
            print("Finished MCMC")

    def print_param_diagnostics(self, pv):
        """
        A function to print nice parameter diagnostics.
        """
        self.df_diagnostics = pd.DataFrame(
            zip(self.lpf.ps.labels, self.lpf.ps.centers, self.lpf.ps.bounds[:, 0], self.lpf.ps.bounds[:, 1], pv,
                self.lpf.ps.centers - pv), columns=["labels", "centers", "lower", "upper", "pv", "center_dist"])
        print(self.df_diagnostics.to_string())
        return self.df_diagnostics

    # def plot_chains(self,labels=None,burn=0,thin=1):
    #    print("Plotting chains")
    #    if labels==None:
    #        labels = self.lpf.ps.descriptions
    #    mcFunc.plot_chains(self.sampler.chain,labels=labels,burn=burn,thin=thin)

    # def plot_corner(self,labels=None,burn=0,thin=1,title_fmt='.5f',**kwargs):
    #    if labels==None:
    #        labels = self.lpf.ps.descriptions
    #    self.fig = mcFunc.plot_corner(self.sampler.chain,labels=labels,burn=burn,thin=thin,title_fmt=title_fmt,**kwargs)


# def sample(df_chain, N=500):
#     spectra = []
#     for i in range(N):
#         c = df_chain.iloc[np.random.randint(len(df_chain))].values[0:4]
#         spectra.append(self.lpf.data_target['f'] - LCS.lpf.compute_model(c))
#     std = np.std(spectra, axis=0)  # np.std(spectra,axis=0)
#     return std


class Chi2FunctionVsiniPolynomial(object):
    def __init__(self, w, f1, e1, f2, e2, maxvsini):
        self.w = w
        self.data_target = {'f': f1,
                            'e': e1}
        self.data_ref = {'f': f2,
                         'e': e2}
        self.priors = [UP(0., maxvsini, 'vsini', '$v \sin i$', priortype="model"),
                       UP(-1e10, 1e10, 'c0', 'c_0', priortype="model"),
                       UP(-1e10, 1e10, 'c1', 'c_1', priortype="model"),
                       UP(-1e10, 1e10, 'c2', 'c_2', priortype="model"),
                       UP(-1e10, 1e10, 'c3', 'c_3', priortype="model"),
                       UP(-1e10, 1e10, 'c4', 'c_4', priortype="model"),
                       UP(-1e10, 1e10, 'c5', 'c_5', priortype="model")]
        self.ps = PriorSet(self.priors)

    def compute_model(self, pv, eps=0.3):
        """
        Multiply reference by polynomial and then rotationally broaden.
        """
        vsini = pv[0]
        coeffs = pv[1:]
        ff_ref = self.data_ref['f'] * np.polynomial.chebyshev.chebval(self.w, coeffs)
        ff_ref = rotbroad_help.broaden(self.w, ff_ref, vsini, u1=eps)
        return ff_ref

    def __call__(self, pv, verbose=False):
        if any(pv < self.ps.pmins) or any(pv > self.ps.pmaxs):
            return np.inf
        flux_model = self.compute_model(pv)
        flux_target = self.data_target['f']
        dummy_error = np.ones(len(flux_target))
        chi2 = stats_help.chi2(flux_target - flux_model, dummy_error, 1, verbose=verbose)
        # print(pv,chi2)
        # log_of_priors = self.ps.c_log_prior(pv)
        return chi2


class FitTargetRefStarVsiniPolynomial(object):
    """
    A class to fit 5 lin-comb spectra together. Note: look at LPFunctionLinComb
    """

    def __init__(self, Chi2FunctionVsiniPolynomial):
        self.chi2f = Chi2FunctionVsiniPolynomial

    def plot_model(self, pv):
        vsini = pv[0]
        coeffs = pv[1:]
        fig, (ax, bx) = plt.subplots(nrows=2, dpi=200, sharex=True, gridspec_kw={'height_ratios': [5, 2]})
        ax.plot(self.chi2f.w, self.chi2f.data_target['f'], color='black', label='Target', lw=1)
        ff = self.chi2f.compute_model(pv)
        ax.plot(self.chi2f.w, np.polynomial.chebyshev.chebval(self.chi2f.w, coeffs))

        ax.plot(self.chi2f.w, ff, color='crimson', label='Reference (vsini={:0.3f}km/s)'.format(vsini), alpha=0.5, lw=1)
        ax.legend(fontsize=10)
        bx.plot(self.chi2f.w, self.chi2f.data_target['f'] - ff, lw=1)
        bx.set_xlabel('Wavelength [A]', fontsize=12)
        bx.set_ylabel('Residual', fontsize=12)

        title = '$\chi^2$={}, coeffs={}'.format(self.chi2f(pv), coeffs)
        ax.set_title(title, fontsize=10)
        for xx in (ax, bx):
            utils.ax_apply_settings(xx, ticksize=10)
        fig.subplots_adjust(hspace=0.05)

    def minimize_AMOEBA(self, centers=[0.01, 0., 0., 0., 0., 0., 1.]):
        # print('Performing first Chebfit')
        # centers_coeffs = np.poly(self.chi2f.w,self.chi2f.data_target['f']-self.chi2f.data_ref['f']+1.,5)
        centers_coeffs = np.polynomial.chebyshev.chebfit(self.chi2f.w,
                                                         self.chi2f.data_target['f'] - self.chi2f.data_ref['f'] + 1., 5)

        # print('Found centers:',centers_coeffs)

        # ####################################################
        # CHANGING 20190629
        centers = [1.] + list(centers_coeffs)
        # centers = [0.5]+list(centers_coeffs)
        # ####################################################

        # print('With CHI',self.chi2f(centers))
        # print(len(centers),len(centers_coeffs))
        # random = self.lpf.ps.random
        # centers = np.array(self.lpf.ps.centers)

        # self.res = scipy.optimize.minimize(self.chi2f,centers,method='Nelder-Mead',tol=1e-1,
        #                           options={'maxiter': 1000, 'maxfev': 2000, 'disp': True})
        self.res = scipy.optimize.minimize(self.chi2f, centers, method='Powell', tol=1e-1,
                                           options={'maxiter': 1000, 'maxfev': 2000, 'disp': True})
        # self.res = scipy.optimize.minimize(self.chi2f,centers,method='Nelder-Mead',tol=1e-7,
        #                           options={'maxiter': 10000, 'maxfev': 5000, 'disp': True})

        self.min_pv = self.res.x

    def minimize_PyDE(self, npop=100, de_iter=200, mc_iter=1000, mcmc=True, threads=8, maximize=False, plot_priors=True,
                      sample_ball=False):
        """
        Minimize using the PyDE
        
        NOTES:
        https://github.com/hpparvi/PyDE
        """
        centers = np.array(self.chi2f.ps.centers)
        print("Running PyDE Optimizer")
        self.de = pyde.de.DiffEvol(self.chi2f, self.chi2f.ps.bounds, npop,
                                   maximize=False)  # we want to maximize the likelihood
        self.min_pv, self.min_pv_chi2val = self.de.optimize(ngen=de_iter)
        print("Optimized using PyDE")
        print("Final parameters:")
        self.print_param_diagnostics(self.min_pv)
        # self.lpf.ps.plot_all(figsize=(6,4),pv=self.min_pv)
        # print("LogLn value:",self.min_pv_lnval)
        # print("Log priors",self.lpf.ps.c_log_prior(self.min_pv))
        if mcmc:
            print("Running MCMC")
            self.sampler = emcee.EnsembleSampler(npop, self.chi2f.ps.ndim, self.chi2f, threads=threads)
            print("MCMC iterations=", mc_iter)
            for i, c in enumerate(self.sampler.sample(self.de.population, iterations=mc_iter)):
                print(i, end=" ")
            print("Finished MCMC")


def chi2spectraPolyVsini(ww, H1, H2, rv1=None, rv2=None, plot=False, verbose=False, maxvsini=30., order=101, deblazed=False):
    """
    INPUT:
        ww - wavelength grid to interpolate on (array)
        H1 - target spectrum (HPFSpectrum object)
        H2 - reference spectrum (HPFSpectrum object)
        rv1 - radial velocity H1 km/s (float)
        rv2 - radial velocity H2 km/s (float)
        plot - (boolean)
        verbose - print additional info (boolean)

    OUTPUT:
        chi2 - chi2 values for the comparison
        vsini - 
        coeffs - 
        
    EXAMPLE:
        H1 = HPFSpectrum(df[df.name=='G_9-40'].filename.values[0])
        H2 = HPFSpectrum(df[df.name=='AD_Leo'].filename.values[0])

        wmin = 10280.
        wmax = 10380.
        ww = np.arange(wmin,wmax,0.01)
        chi2spectraPolyVsini(ww,H1,H2,rv1=14.51,plot=True)
        
    """
    ff1, ee1 = H1.resample_order(ww, order=order, deblazed=deblazed)
    ff2, ee2 = H2.resample_order(ww, order=order)
    plt.close('all')
    plt.plot(ww, ff1, label='H1')
    plt.plot(ww, ff2, label='H2')
    # plt.title(H2.basename)
    # plt.savefig(f"/Users/tehan/Documents/NEID_archive/14020_Spectra/{H2.basename.replace('.fits','.png')}")
    C = Chi2FunctionVsiniPolynomial(ww, ff1, ee1, ff2, ee2, maxvsini)
    FTRSVP = FitTargetRefStarVsiniPolynomial(C)
    FTRSVP.minimize_AMOEBA()
    vsini = FTRSVP.min_pv[0]
    coeffs = FTRSVP.min_pv[1:]
    chi2 = C(FTRSVP.min_pv)

    if plot:
        FTRSVP.plot_model(FTRSVP.min_pv)

    return chi2, vsini, coeffs


def chi2spectraPolyLoop(ww, H1, Hrefs, plot_all=False, plot_chi=True, verbose=True, maxvsini=30., order=101, deblazed=False):
    """
    Calculate chi square - target and list of reference spectra
    
    INPUT:
        ww - wavelength grid to interpolate on (array)
        H1 - target spectrum (HPFSpectrum object)
        Hrefs - reference spectra (HPFSpectraList object)
        plot_all - creates many additional plots (boolean)
        plot_chi = plot chi2 H1 vs other stars (boolean)
        verbose - print additional info (boolean)
    
    OUTPUT:
        df - dataframe of all reference stars sorted by chi2 (chi2, poly_params, and vsini values)
        df_best - dataframe with 5 best reference stars (chi2, poly_params, and vsini values)
        Hrefs_best - 5 best fitting reference stars
        
    EXAMPLE:
        
    """
    chis = []
    poly_params = []
    vsinis = []
    for i, H2 in enumerate(Hrefs):
        if i == 0:  # SEJ
            print('First step: Matching target star to all library stars')
            print("##################")

        chi, vsini, p = chi2spectraPolyVsini(ww, H1, H2, plot=plot_all, maxvsini=maxvsini, order=order, deblazed=deblazed)
        if verbose:
            print(
                '{:3d}/{:2d}, Target = {:18s} Library Star = {:18s} chi2 = {:6.3f}'.format(i + 1, len(Hrefs), H1.object,
                                                                                           H2.object, chi))
        chis.append(chi)
        poly_params.append(p)
        vsinis.append(vsini)

    refnames = [H.object for H in Hrefs]
    df = pd.DataFrame(list(zip(refnames, chis, poly_params, vsinis)),
                      columns=['OBJECT_ID', 'chi2', 'poly_params', 'vsini'])
    df = df.sort_values('chi2')
    df = df.reset_index(drop=False)
    df_best = df[0:5]
    Hrefs_best = neidspec.NEIDSpecList(np.array(Hrefs)[df_best['index'].values]);  # SEJ

    if plot_chi:
        fig, ax = plt.subplots(dpi=200)
        ax.plot(df.chi2, lw=0, marker='h')
        ax.set_xticks(range(len(chis)))
        ax.set_ylabel('$\chi^2$')
        utils.ax_apply_settings(ax)
        ax.set_xticklabels(df['OBJECT_ID'])
        ax.tick_params('x', labelsize=8)
        ax.set_title('{} vs other stars'.format(H1.object))
        for lab in ax.get_xticklabels():
            lab.set_rotation(90.)
        ax.set_yscale('log')
        # x.set_ylim(0.01,1e3)
    return df, df_best, Hrefs_best


def weighted_value(values, weights):
    """
    Array of weighted values
    """
    return np.dot(values, weights)


def run_specmatch(Htarget, Hrefs, ww, v, df_library, df_target=None, plot=True, savefolder='out/',
                  maxvsini=30., calibrate_feh=True, scaleres=1., order=101, deblazed=False, mode='HR', save_plot_data=False):
    """
    Second chi2 loop, creates composite spectrum 
    
    INPUT:
        Htarget - target spectrum (HPFSpectrum object)
        Hrefs - reference spectra (HPFSpectraList object)
        ww - wavelength grid to interpolate on (array)
        v - velocities in km/s to use for absolute RV consideration (array)
        df_library - dataframe with info on Teff/FeH/logg for the library stars
        df_target - dataframe with target parameter info
        plot - save SpecMatch plots (boolean)
        savefolder - output directory name (String)
        scaleres - amount to scale residuals from composite spectrum (default 1)
    
    OUTPUT:
        stellar parameters teff, feh, logg, vsini, and their errors
        df_chi_total, LCS  
    
    EXAMPLE:
        
    """
    print('##################')
    print('Saving results to {}'.format(savefolder))
    print('##################')
    utils.make_dir(savefolder)
    targetname = Htarget.object
    ##############################

    print('##################')
    print('Running Chi2 loop')
    print('##################')
    ##############################
    # STEP 1: Chi2 Loop
    df_chi, df_chi_best, Hbest = chi2spectraPolyLoop(ww, Htarget, Hrefs, plot_all=False, verbose=True,
                                                     maxvsini=maxvsini, order=order, deblazed=deblazed)
    ##############################
    # Combine best data
    # print(df_chi_best['OBJECT_ID'])
    # print(df_library['OBJECT_ID'])
    df_chi_best_total = pd.merge(df_chi_best, df_library, on='OBJECT_ID', how='inner')
    # print(df_chi_best_total['OBJECT_ID'])
    df_chi_total = pd.merge(df_chi, df_library, on='OBJECT_ID')
    if plot:
        plot_chi_teff_feh_logg_panel(df_chi_total.chi2,
                                     df_chi_total.Teff,
                                     df_chi_total['[Fe/H]'],
                                     df_chi_total['log(g)'],
                                     savename=savefolder + targetname + '_chi2_3panel.png')
        plot_teff_feh_logg_corr_panel(df_chi_total.Teff.values,
                                      df_chi_total['[Fe/H]'].values,
                                      df_chi_total['log(g)'].values,
                                      df_chi_total.chi2.values,
                                      e_teff=df_chi_total['e_Teff'],
                                      e_feh=df_chi_total['e_[Fe/H]'],
                                      e_logg=df_chi_total['e_log(g)'],
                                      savename=savefolder + targetname + '_chi2_corrpanel.png')
    if df_target is not None:
        teff_known = df_target.Teff.values[0]
        tefferr_known = df_target.e_Teff.values[0]
        feh_known = df_target['[Fe/H]'].values[0]
        feherr_known = df_target['e_[Fe/H]'].values[0]
        logg_known = df_target['log(g)'].values[0]
        loggerr_known = df_target['e_log(g)'].values[0]
    else:
        teff_known = np.nan
        tefferr_known = np.nan
        feh_known = np.nan
        feherr_known = np.nan
        logg_known = np.nan
        loggerr_known = np.nan

    print('##################')
    print('Performing Linear Combination Optimization')
    print('##################')

    ##############################
    # STEP 2 LINEAR COMBINATION
    ##############################
    f1, e1, ffrefs, eerefs = get_data_ready(Htarget, Hbest, ww, v, polyvals=df_chi_best.poly_params.values,
                                            vsinis=df_chi_best.vsini.values, order=order, deblazed=deblazed)
    L = LPFunctionLinComb(ww, f1, e1, ffrefs, eerefs)
    LCS = FitLinCombSpec(L, df_chi_best_total.Teff.values,
                         df_chi_best_total['[Fe/H]'].values,
                         df_chi_best_total['log(g)'].values,
                         df_chi_best_total['vsini'].values,
                         teff_known=teff_known,
                         tefferr_known=tefferr_known,
                         feh_known=feh_known,
                         feherr_known=feherr_known,
                         logg_known=logg_known,
                         loggerr_known=loggerr_known,
                         targetname=targetname,
                         calibrate_feh=calibrate_feh)
    LCS.minimize_PyDE(mcmc=False)
    print(LCS.min_pv)
    # LCS.plot_model(LCS.min_pv)# SEJ
    if plot:
        LCS.plot_model_with_components(LCS.min_pv, names=df_chi_best['OBJECT_ID'].values, title="", mode=mode,
                                       savename=savefolder + targetname + '_compositecomparison.png', scaleres=scaleres)
    if save_plot_data:
        LCS.plot_model_with_component_save(LCS.min_pv, names=df_chi_best['OBJECT_ID'].values, title="", mode=mode,
                                       datafile=savefolder + targetname + '_compositecomparison.pkl', scaleres=scaleres)

    teff = LCS.teff
    feh = LCS.feh
    logg = LCS.logg
    vsini = LCS.vsini
    teff_delta = teff - teff_known
    feh_delta = feh - feh_known
    logg_delta = logg - logg_known

    # Saving to pickle file
    weights = LCS.lpf.get_pv_all(LCS.min_pv)
    results = {'teff': teff,
               'logg': logg,
               'feh': feh,
               'vsini': vsini,
               'weights': weights}
    savefile = open(savefolder + targetname + '_results.pkl', "wb")
    pickle.dump(results, savefile)
    savefile.close()
    print('Saved results to {}'.format(savefolder + targetname + '_results.pkl'))

    print('##################')
    print('Saving Chi2 dataframe to {}'.format(savefolder + targetname + '_chi2results.csv'))
    print('##################')
    df_chi_total.to_csv(savefolder + targetname + '_chi2results.csv', index=False)
    df_chi_best_total['weights'] = weights
    df_chi_best_total.to_csv(savefolder + targetname + '_chi2results_best.csv', index=False)
    return teff, feh, logg, vsini, teff_delta, feh_delta, logg_delta, df_chi_total, LCS


def plot_chi_teff_feh_logg_panel(chis, teff, feh, logg, savename='chi2panel.pdf', fig=None, title=''):
    """
    3panel plot chi2 vs Teff, FeH, logg for 5 best fit stars among all library stars
    
    INPUT:
        chis - (array)
        teff - (array)
        feh - (array)
        logg - (array)
        savename - directory name to save plot (str)
        title - plot title (str)

    OUTPUT:
        saves 3panel plot
            
    EXAMPLE:
        plot_chi_teff_feh_logg_panel(df_chi.chi2,df_chi.Teff,df_chi['[Fe/H]'],df_chi['log(g)'],savename='chi2.pdf')
    """
    if fig is None:
        fig, (ax, bx, cx) = plt.subplots(dpi=200, ncols=3, sharey=True, figsize=(6, 2))
    df = pd.DataFrame(list(zip(chis, teff, feh, logg)), columns=['chi2', 'teff', 'feh', 'logg'])
    df_best = df.sort_values('chi2').reset_index(drop=True)[0:5]
    ax.plot(teff, chis, marker='o', lw=0)
    ax.plot(df_best.teff, df_best.chi2, color='crimson', marker='h', lw=0)
    # ax.set_xlim(2950.,4050)
    # ax.set_xticks(np.arange(3000,4100,200))

    ax.set_yscale('log')
    ax.set_ylabel('$\chi^2$', fontsize=10, labelpad=2)
    ax.set_xlabel('$T_{\mathrm{eff}}$ [K]', fontsize=10, labelpad=2)
    bx.set_xlabel('[Fe/H] ', fontsize=10, labelpad=2)
    cx.set_xlabel('$\log(g)$', fontsize=10, labelpad=2)

    bx.plot(feh, chis, marker='o', lw=0)
    bx.plot(df_best.feh, df_best.chi2, color='crimson', marker='h', lw=0)
    # bx.set_xlim(-0.55,0.55)
    # bx.set_xticks(np.arange(-0.5,0.55,0.25))

    cx.plot(logg, chis, marker='o', lw=0)
    cx.plot(df_best.logg, df_best.chi2, color='crimson', marker='h', lw=0)
    # cx.set_xlim(4.7,5.1)

    for xx in (ax, bx, cx):
        utils.ax_apply_settings(xx, ticksize=7)
        xx.tick_params(pad=2)
        # xx.set_ylim(3e3,1.5e5)
    bx.set_title(title, fontsize=14)
    fig.subplots_adjust(wspace=0.1, right=0.97, left=0.08, bottom=0.15, top=0.95)
    fig.savefig(savename, dpi=200)
    print('Saved to: {}'.format(savename))


def plot_teff_feh_logg_corr_panel(teff, feh, logg, chis, e_teff=None, e_feh=None, e_logg=None, fig=None, ax=None,
                                  bx=None,
                                  savename='corr_panel.pdf', scale_factor_for_points=1500.):
    """
    Plot of teff vs feh, logg for 5 best fit stars among all library stars
    
    INPUT:
        teff - (array)
        feh - (array)
        logg - (array)
        chis - (array)
        savename - directory name to save plot (str)
        title - plot title (str)

    OUTPUT:
        saves correlation plot
        
    """
    if ax is None and bx is None and fig is None:
        fig, (ax, bx) = plt.subplots(ncols=2, dpi=200, sharey=True, figsize=(10, 3))
    colors = utils.get_cmap_colors(p=chis)
    df_chi = pd.DataFrame(zip(teff, feh, logg, chis), columns=['Teff', '[Fe/H]', 'log(g)', 'chi2'])
    df_chi = df_chi.sort_values('chi2').reset_index(drop=True)
    df_chi_best = df_chi[0:5]

    # First plot:
    ax.scatter(df_chi['[Fe/H]'], df_chi.Teff, s=scale_factor_for_points / df_chi.chi2.values ** 2., color='black')
    ax.scatter(df_chi_best['[Fe/H]'], df_chi_best.Teff, lw=0, color='crimson',
               s=scale_factor_for_points / df_chi_best.chi2.values ** 2.)

    if e_feh is not None and e_logg is not None and e_teff is not None:
        median_xerr = np.median(e_feh)
        median_yerr = np.median(e_teff)
        ax.errorbar(-0.4, 3950, xerr=median_xerr, yerr=median_yerr, capsize=3, elinewidth=0.5, mew=0.5, color='black')
        ax.text(-0.40 + 0.0, 3820, 'Median error', fontsize=10, horizontalalignment='center')

    # Second plot
    bx.scatter(df_chi['log(g)'], df_chi.Teff, s=scale_factor_for_points / df_chi.chi2.values ** 2., color='black',
               label='Library spectra:\n(radius$\propto 1/\chi^2$)')
    bx.scatter(df_chi_best['log(g)'], df_chi_best.Teff, lw=0, color='crimson',
               s=scale_factor_for_points / df_chi_best.chi2.values ** 2., label='5 Best-fit spectra')
    bx.legend(loc='upper right', fontsize=9, frameon=False)

    if e_feh is not None and e_logg is not None and e_teff is not None:
        median_xerr = np.median(e_logg)
        bx.errorbar(4.68, 3230, xerr=median_xerr, yerr=median_yerr, capsize=3, elinewidth=0.5, mew=0.5, color='black')
        bx.text(4.685, 3100, 'Median error', fontsize=10, horizontalalignment='center')

    ax.set_ylabel('$T_{\mathrm{eff}}$ [K]', fontsize=12, labelpad=2)
    ax.set_xlabel('Fe/H [dex]', fontsize=12, labelpad=1)
    bx.set_xlabel('$\log (g)$ [dex]', fontsize=12, labelpad=1)

    fig.subplots_adjust(wspace=0.05)

    for xx in (ax, bx):
        utils.ax_apply_settings(xx, ticksize=9)
        xx.grid(lw=0.4, alpha=0.3)
        utils.ax_set_linewidth(xx, 1.3)

    fig.savefig(savename, dpi=200)
    print('Saved to: {}'.format(savename))


def summarize_values_from_orders(files_pkl, targetname):
    """
    Summarize values from different orders from SpecMatch analysis

    INPUT:
        Name of pickle files

    OUTPUT:
        saves two .csv files:
            1) a .csv file with the Teff, Fe/H, and logg values from all of the orders
            2) a .csv file with the median and the standard deviation from different orders

    Example:
        files = sorted(glob.glob('20200209_ad_leos/AD_Leo/*/*.pkl'))
        summarize_values_from_orders(files,'AD_Leo')
    """
    # basedir = files_pkl[0].split(os.sep)[0:2]
    # basedir = os.path.join(*files[0].split(os.sep)[0:-2])
    savefolder = os.path.join(*files_pkl[0].split(os.sep)[0:-2])
    target = targetname
    params = ['teff', 'logg', 'feh', 'vsini']
    results = []
    for filename in files_pkl:
        f = open(filename, 'rb')
        res = pickle.load(f)
        results.append(res)
        f.close()
    df = pd.DataFrame(results)
    df['filenames'] = files_pkl
    medians = df.median(axis=0).values
    stds = df.std(axis=0).values

    df_med = pd.DataFrame(list(zip(params, medians, stds)), columns=['parameters', 'median', 'std'])

    df.to_csv(os.sep + savefolder + os.sep + target + '_overview.csv', index=False)
    df_med.to_csv(os.sep + savefolder + os.sep + target + '_med.csv', index=False)

    print('Saved to {}'.format(savefolder + os.sep + target + '_overview.csv'))
    print('Saved to {}'.format(savefolder + os.sep + target + '_med.csv'))
    return df, df_med


def run_specmatch_for_orders(targetfile, targetname, outputdirectory='specmatch_results', HLS=None,
                             path_df_lib=config.PATH_LIBRARY_DB, orders=['55', '101', '102', '103'],
                             maxvsini=30., calibrate_feh=True, scaleres=1., deblazed=False, mode='HR', save_plot_data=False, add_vsini=10):
    """
    run neidspecmatch for a given target file and orders
    
    INPUT:
        targetfile - name of target file
        targetname - target name, queried via simbad or tic ('GJ_251' or TIC 68581262)
        outputdirectory - folder to save overall results and plots
        HLS - refence stars as an HPFSpecList object, defaults to normal library
        path_df_lib - path to .csv file containing info on Teff/FeH/logg for all library stars
                    - defaults to config.PATH_LIBRARY_DB
        orders - hpf orders to run (orders 4, 5, 6, 14, 15, 16, and 17
                    recommended as they are the cleanest orders with minimal tellurics)
        maxvsini - maximum vsini to consider (default = 30 km/s)
    
    OUTPUT:
        result files will be saved to outputdirectory
    
    EXAMPLE:
        filename = '../input/20201020_hpf_gto_targets/Slope-20200114T091114_R01.optimal.fits'
        targetname = 'GJ_251'
        HLS = hpfspec.HPFSpecList(filelist=library_fitsfiles)
        outputdir = '20201020_hpf_gto_targets/GJ_251'
        run_specmatch_for_orders(filename, targetname , outputdir, HLS)
    
    NOTES:
        targetname will be queried via simbad or tic which saves a configuration file to target config directory
    
    """
    # Target data
    Htarget = neidspec.NEIDSpectrum(targetfile, targetname=targetname, add_vsini=add_vsini)
    print('Reading Library DataBase from: {}'.format(path_df_lib))
    df_lib = pd.read_csv(path_df_lib)

    # Reference data
    if HLS is None:
        print('No HLS supplied, defaulting to default library')
        HLS = neidspec.NEIDSpecList(filelist=config.LIBRARY_FITSFILES)
        Hrefs = HLS.splist

    # Run spectral matching algorithm for first two orders
    # in principle we should run all orders, just first two as an example
    vsinis = []
    for o in orders:
        # Htarget = neidspec.NEIDSpectrum(targetfile, targetname=targetname, add_vsini=add_vsini)
        print("##################")
        print("Order {}".format(o))
        print("##################")
        wmin = config.BOUNDS[o][0]  # Lower wavelength bound in A
        wmax = config.BOUNDS[o][1]  # Upper wavelength bound in A
        ww = np.arange(wmin, wmax, 0.01)  # Wavelength array to resample to
        v = np.linspace(-175, 175, 2501)  # Velocities in km/s to use for absolute RV consideration
        savefolder = '{}/{}_{}/'.format(outputdirectory, Htarget.object, o)  # foldername to save

        #############################################################
        # Run specmatch for order 
        #############################################################
        t, f, l, vis, te, fe, le, df_chi, LCS = run_specmatch(Htarget,  # Target class
                                                              HLS.splist,  # Library spectra
                                                              ww,  # Wavelength to resample to
                                                              v,  # velocity range to use for absolute rv
                                                              df_lib,
                                                              # dataframe with info on Teff/FeH/logg for the library stars
                                                              savefolder=savefolder,
                                                              maxvsini=maxvsini,
                                                              calibrate_feh=calibrate_feh,
                                                              scaleres=scaleres,
                                                              order=int(o),
                                                              deblazed=deblazed,
                                                              mode=mode,
                                                              save_plot_data=save_plot_data)
        vsinis.append(vis)
    return vsinis

def plot_crossvalidation_results_1d(order, df_crossval, savefolder):
    """
    
    """
    print('plotting 1D crossval')
    fig, (ax, bx, cx) = plt.subplots(nrows=3, sharex=True, dpi=200)
    x = range(len(df_crossval))
    label = 'std={:0.2f}K'.format(np.std(df_crossval.d_teff))
    print(label)
    ax.plot(df_crossval.d_teff, 'k.', markersize=8)
    ax.set_ylim(ax.get_ylim()[0] * 1.6, ax.get_ylim()[1] * 2.2)

    label = 'std={:0.2f}dex'.format(np.std(df_crossval.d_feh))
    print(label)
    bx.plot(df_crossval.d_feh, 'k.', markersize=8)
    bx.set_ylim(bx.get_ylim()[0] * 2.0, bx.get_ylim()[1] * 2.2)

    label = 'std={:0.2f}dex'.format(np.std(df_crossval.d_logg))
    print(label)
    cx.plot(df_crossval.d_logg, 'k.', markersize=8)
    cx.set_ylim(cx.get_ylim()[0] * 2.2, cx.get_ylim()[1] * 2.2)
    np.savetxt(savefolder + 'crossval_std.csv', np.array([[np.std(df_crossval.d_teff), np.std(df_crossval.d_feh),
                                                           np.std(df_crossval.d_logg)]]), delimiter=',')
    for xx in (ax, bx, cx):
        utils.ax_apply_settings(xx)
        xx.tick_params(labelsize=9, pad=2)
        xx.grid(lw=0.3)
        # xx.legend(bbox_to_anchor=(1.,1.),fontsize=12)
    fig.subplots_adjust(hspace=0.05)
    ax.set_ylabel('$\Delta$Teff [K]', fontsize=12, labelpad=-4)
    bx.set_ylabel('$\Delta$Fe/H [K]', fontsize=12, labelpad=-2)
    cx.set_ylabel('$\Delta$logg [K]', fontsize=12, labelpad=-2)
    cx.set_xlabel('Library spectrum #', fontsize=12)
    ax.set_title('Library Performance (NEID Order {})'.format(order))
    plt.savefig('{}/crossvalidation_o{}_plot1D.png'.format(savefolder, order))


def plot_crossvalidation_results_2d(order, df_crossval, savefolder):
    """
    """
    print('plotting 2D crossval')
    dd = df_crossval[["d_teff", "d_feh", "d_logg"]]
    fig, axx = plt.subplots(nrows=3, ncols=3, dpi=200, figsize=(6, 6))

    for xx in [axx[0, 1], axx[0, 2], axx[1, 2]]:
        xx.axes.set_axis_off()

    labels = ["$\Delta T_{\mathrm{eff}}$", "$\Delta$[Fe/H]", "$\Delta \log g$"]
    xlabels = ["$\Delta T_{\mathrm{eff}}$ [K]", "$\Delta$[Fe/H] [dex]", "$\Delta \log g$ [dex]"]

    # xlims = [xx.get_xlim() for xx in [axx[1,0],axx[2,0],axx[2,1]]]
    xlims = []
    diag = [axx[0, 0], axx[1, 1], axx[2, 2]]
    for i, xx in enumerate(diag):
        xx.set_title(labels[i], fontsize=12)
        _y, _x, _ = xx.hist(dd.iloc[:, i].values, color="black", histtype="step", bins=7,
                            range=[dd.iloc[:, i].values.min(), dd.iloc[:, i].values.max()])
        g = astropy.modeling.models.Gaussian1D(stddev=np.std(dd.iloc[:, i].values))
        xmax = np.max(np.abs(dd.iloc[:, i].values))
        x = np.linspace(-xmax, xmax, 1000)
        y = (g(x) / (np.max(g(x)))) * (np.max(_y))
        xx.plot(x, y, color="crimson", lw=1, ls="--")

        if i == 0:
            xx.xaxis.set_visible(False)
            xx.yaxis.set_visible(False)
            # xx.set_xlim(xlims[0][0],xlims[0][1])
        if i == 1:
            xx.xaxis.set_visible(False)
            xx.yaxis.set_visible(False)
            # xx.set_xlim(xlims[2][0],xlims[2][1])
        if i == 2:
            xx.yaxis.set_visible(False)
        xlims.append(xx.get_xlim())

    axx[1, 0].plot(df_crossval.d_teff.values, df_crossval.d_feh.values, marker="o", lw=0, color="k", alpha=0.8)
    axx[1, 0].set_xlim(xlims[0][0], xlims[0][1])
    axx[2, 0].plot(df_crossval.d_teff.values, df_crossval.d_logg.values, marker="o", lw=0, color="k", alpha=0.8)
    axx[2, 0].set_xlim(xlims[0][0], xlims[0][1])
    axx[2, 1].plot(df_crossval.d_feh.values, df_crossval.d_logg.values, marker="o", lw=0, color="k", alpha=0.8)
    axx[2, 1].set_xlim(xlims[1][0], xlims[1][1])

    axx[2, 1].yaxis.set_visible(False)
    axx[1, 0].xaxis.set_visible(False)

    axx[1, 0].set_ylabel(xlabels[1], fontsize=12, labelpad=3)
    axx[2, 0].set_ylabel(xlabels[2], fontsize=12, labelpad=-2)
    axx[2, 0].set_xlabel(xlabels[0], fontsize=12)
    axx[2, 1].set_xlabel(xlabels[1], fontsize=12)
    axx[2, 2].set_xlabel(xlabels[2], fontsize=12)

    for xx in axx.flatten():
        utils.ax_apply_settings(xx, ticksize=10)
        xx.grid(lw=0)
        xx.tick_params(pad=1)

    fig.subplots_adjust(wspace=0.02, hspace=0.02)
    plt.savefig('{}/crossvalidation_o{}_plot2D.png'.format(savefolder, order))


def run_crossvalidation_for_orders(order, df_lib=config.PATH_LIBRARY_DB, HLS=None,
                                   outputdir=config.PATH_LIBRARY_CROSSVAL,
                                   plot_results=True,
                                   calibrate_feh=True,
                                   scaleres=1.):
    """
    Run cross validation for a given order
    
    Here, cross validation is performed using a "leave one out" approach.
    We loop through the full stellar library leaving one library star out,
    where we try and recover the stellar parameters for that spectrum and compare it with the known values
    
    INPUT:
        order - order to run cross validation for (orders 4, 5, 6, 14, 15, 16, and 17
                    recommended as they are the cleanest orders with minimal tellurics)
        df_lib - dataframe with info on Teff/FeH/logg for the library stars
        HLS - refence stars as an HPFSpecList object
        outputdir - folder to save overall results and plots
        plot_results - plots figures summarizing the cross validation results
        scaleres - amount to scale residuals from composite spectrum (default 1)
    
    OUTPUT:
        returns df_crossval dataframe with columns 'teff','feh','logg','vsini',
            'd_teff','d_feh','d_logg' where d_*param* is the difference between the known value
             and the recovered value from the cross validation
        df_crossval saved to csv in outputdir
    
    EXAMPLE:
        order = '17' # recommended order
        
        LIBRARY_DIR = '../library/20201008_specmatch_nir/'
        
        df_lib = pd.read_csv(LIBRARY_DIR+'20201008_specmatch_nir.csv')
        library_fitsfiles = glob2.glob(LIBRARY_DIR+'FITS/*/*.fits')
        HLS = hpfspec.HPFSpecList(filelist=library_fitsfiles)
        
        outputdir = 'cross_validation_{}'.format(order)
        
        run_crossvalidation_for_orders(order, df_lib,HLS, outputdir)
        
    
    NOTES:
        
    
    """
    wmin = config.BOUNDS[order][0]
    wmax = config.BOUNDS[order][1]
    ww = np.arange(wmin, wmax, 0.01)
    v = np.linspace(-175, 175, 2501)
    res = []
    obj_names = []

    outputdir = '{}/o{}_crossval'.format(outputdir, order)

    # Reference data
    if HLS is None:
        print('No HLS supplied, defaulting to default library')
        HLS = neidspec.NEIDSpecList(filelist=config.LIBRARY_FITSFILES)

    # Looping over every star in the library
    print(len(df_lib))
    print(HLS)
    for i in range(len(df_lib)):
        print(i)
        # Target data
        Htarget = HLS.splist[i]
        print(Htarget.object)
        df_target = df_lib[df_lib['OBJECT_ID'] == Htarget.object]
        # Removing the target star from the library
        Hrefs = np.delete(np.array(HLS.splist), i)
        # Run specmatch without the target star in the library
        _res = run_specmatch(Htarget, Hrefs, ww, v, df_lib, df_target, plot=True,
                             savefolder='{}/plots/'.format(outputdir),
                             calibrate_feh=calibrate_feh,
                             scaleres=scaleres, order=int(order))
        res.append(_res)
        obj_names.append(Htarget.object)

    # Collect all of the results and save
    df_crossval = pd.DataFrame(res, columns=['teff', 'feh', 'logg', 'vsini', 'd_teff', 'd_feh', 'd_logg', '_', '__'])
    df_crossval = df_crossval[['teff', 'feh', 'logg', 'vsini', 'd_teff', 'd_feh', 'd_logg']]
    df_crossval['teff_true'] = df_crossval.teff - df_crossval.d_teff
    df_crossval['feh_true'] = df_crossval.feh - df_crossval.d_feh
    df_crossval['logg_true'] = df_crossval.logg - df_crossval.d_logg
    df_crossval['targetname'] = obj_names

    fig, ax = plt.subplots(dpi=200)
    plot_crossval_feh_delta_feh(df_crossval['feh_true'].values, df_crossval['d_feh'].values, ax=ax)
    fig.savefig('{}/crossvalidation_feh_delta_feh_o{}.png'.format(outputdir, order), dpi=200)

    result_savename = '{}/crossvalidation_results_o{}.csv'.format(outputdir, order)
    df_crossval.to_csv(result_savename, index=False)
    print('Saved result to: {}'.format(result_savename))

    if plot_results == True:
        plot_crossvalidation_results_1d(order, df_crossval, outputdir)
        plot_crossvalidation_results_2d(order, df_crossval, outputdir)
        plot_crossvalidation_results_main(order, df_crossval, outputdir)

    return df_crossval


def detrend_feh(feh):
    """
    Calibrate trend in (Delta Fe/H = Fe/H_recovered - Fe/H_true) vs Fe/H_true
    
    INPUT:
        recovered feh value from specmatch
        
    OUTPUT:
        calibrated feh after removing trend
        
    NOTES:
        Currently correct for order 5
    """
    # Paramters for order 5
    p1 = -0.28935366
    p0 = -0.00095669
    a = (1. / (p1 + 1.))  # 1.4071697041315936
    b = -(p0 / (p1 + 1.))  # -0.0013462251842456543
    return a * feh + b


def plot_crossvalidation_results_main(order, df_crossval, savefolder):
    """
    Main crossvalidation results plot
    """
    PW = 10.
    PH = 3.
    fig = plt.figure(figsize=(PW, PH), dpi=200)
    gs0 = GridSpec(1, 2)
    gs0.update(top=0.92, bottom=0.11, wspace=0.01, left=0.05, right=0.6)

    gs1 = GridSpec(1, 1)
    gs1.update(top=0.92, bottom=0.11, hspace=0.18, left=0.68, right=0.98)

    ax = plt.subplot(gs0[0, 0])
    bx = plt.subplot(gs0[0, 1])
    cx = plt.subplot(gs1[0, 0])

    ax.plot(df_crossval['feh_true'], df_crossval['teff_true'], marker='o', lw=0, markeredgecolor='black')
    bx.plot(df_crossval['logg_true'], df_crossval['teff_true'], marker='o', lw=0, markeredgecolor='black')
    for i in range(len(df_crossval)):
        _x = [df_crossval['feh_true'].values[i], df_crossval['feh'].values[i]]
        _y = [df_crossval['teff_true'].values[i], df_crossval['teff'].values[i]]
        ax.plot(_x, _y, color='crimson', lw=0.5, zorder=-10)

        _x = [df_crossval['logg_true'].values[i], df_crossval['logg'].values[i]]
        if i == 0:
            bx.plot(_x, _y, color='crimson', lw=0.5, label='Crossvalidation Value', zorder=-10)
        else:
            bx.plot(_x, _y, color='crimson', lw=0.5, zorder=-10)

    ax.set_xlim(-0.6, 0.6)
    ax.set_xlabel('[Fe/H]', labelpad=0)
    ax.set_ylabel('$T_{\mathrm{eff}}$ [K]', labelpad=0)
    bx.set_xlabel('$\log (g)$', labelpad=0)
    ax.minorticks_on()
    bx.minorticks_on()
    bx.yaxis.set_ticks([])
    bx.set_ylim(*ax.get_ylim())
    fig.subplots_adjust(wspace=0.05)
    bx.legend(loc='upper right')
    fig.suptitle('Library Performance (NEID Order {})'.format(order), y=0.98)
    plot_crossval_feh_delta_feh(df_crossval.feh_true.values, df_crossval.d_feh.values, ax=cx)
    fig.savefig('{}/crossvalidation_o{}_main.png'.format(savefolder, order))


def plot_crossval_feh_delta_feh(feh_true, d_feh, ax=None):
    """
    INPUT:
        feh_true - True Fe/H
        d_feh - delta Fe/H = Fe/H_specmatch - Fe/H_True

    EXAMPLE:
        plot_crossval_feh_delta_feh(df_res['[Fe/H]'].values,df_res['d_feh'].values)
    """
    if ax is None:
        fig, ax = plt.subplots(dpi=200)
    ax.plot(feh_true, d_feh, marker='o', lw=0, color='black')

    p = np.polyfit(feh_true, d_feh, deg=1)
    xx = np.linspace(-0.5, 0.5, 200)
    yy = np.polyval(p, xx)
    ax.plot(xx, yy, color='crimson', label='Linear fit\n$p_1$={:0.4f}\n$p_2$={:0.4f}'.format(p[0], p[1]))
    ax.legend(loc='upper right')
    ax.set_xlabel('[Fe/H]', labelpad=0)
    ax.set_ylabel('$\Delta$[Fe/H] = $\mathrm{[Fe/H]}_{\mathrm{Recovered}}$ - $\mathrm{[Fe/H]}_{\mathrm{True}}$')
    ax.minorticks_on()



import os
import numpy as np
from astropy.io import ascii, fits
from astropy import units as u
from astropy import constants
from astroquery.simbad import Simbad
from astropy.table import Table, Column
from astropy.coordinates import SkyCoord
from tqdm import trange
DIRNAME = os.path.dirname(os.path.dirname(__file__))

def load_Mann(file='asu.fit'):
    Mann = fits.open(f'{DIRNAME}/library/{file}')
    return Mann[1].data


def load_Yee(file='apjaa58eat6_mrt.txt'):
    Yee = ascii.read(f'{DIRNAME}/library/{file}')
    return Yee

#from Caleb
def calclogg(mass, masserr, radius, raderr):
    mass = np.array(mass, copy=True) * u.Msun
    masserr = np.array(masserr, copy=True) * u.Msun
    radius = np.array(radius, copy=True) * u.Rsun
    raderr = np.array(raderr, copy=True) * u.Rsun
    val = np.log10((constants.G * mass * radius ** (-2)).cgs.value)
    valerr = np.sqrt(
        ((mass ** (-2.) * masserr ** 2. + 4. * raderr ** 2. * radius ** (-2.)) * np.log(10.) ** (-2.)).value)
    return val, valerr

def combine_Mann_Yee():
    Mann = load_Mann()
    Yee = load_Yee()
    print(Mann[-20])
    combined = Yee
    combined.add_column(Column(name='Source', data=['Yee'] * len(Yee)))
    for i in range(len(Mann)):
        logg, e_logg = calclogg(Mann[i][51], Mann[i][52], Mann[i][49], Mann[i][50])
        combined.add_row({'Name': f'{Mann[i][3]}', 'Teff': f'{Mann[i][47]}', 'e_Teff': f'{Mann[i][48]}',
                                        'R*': f'{Mann[i][49]:.3f}', 'e_R*': f'{Mann[i][50]:.3f}', 'log(g)': f'{logg:.3f}',
                                        'e_log(g)': f'{e_logg:.3f}', '[Fe/H]': f'{Mann[i][14]:.3f}', 'e_[Fe/H]': f'{Mann[i][15]:.3f}',
                                        'M*': f'{Mann[i][51]:.3f}', 'e_M*': f'{Mann[i][52]:.3f}',
                                        'logA': f'{Mann[i][53]:.3f}', 'e_logA': f'{Mann[i][54]:.3f}', 'plx': 0.,
                                        'Vmag': f'{Mann[i][19]:.3f}', 'Notes': 'N/A', 'Source': 'Mann'})
    ra = []
    dec = []
    for j in trange(len(combined)):
        try:
            result_table = Simbad.query_object(combined['Name'][j])
            ra_deg = result_table['RA']
            dec_deg = result_table['DEC']
            coord = SkyCoord(ra=ra_deg[0], dec=dec_deg[0], unit=(u.hourangle, u.deg))
            ra.append(coord.ra.deg)
            dec.append(coord.dec.deg)
        except TypeError:
            print(j, combined[j]['Name'])
            ra.append(-100.)
            dec.append(-100.)
    combined.add_column(Column(name='RA', data=ra))
    combined.add_column(Column(name='Dec', data=dec))
    return combined

def find_star(name, combined=None):
    custom_simbad = Simbad()
    custom_simbad.add_votable_fields('rvz_radvel')
    result_table = custom_simbad.query_object(name)
    radial_velocity = result_table['RVZ_RADVEL'][0]
    # print(f"Radial Velocity of {name}: {radial_velocity} km/s")
    ra_deg = result_table['RA']
    dec_deg = result_table['DEC']
    coord = SkyCoord(ra=ra_deg[0], dec=dec_deg[0], unit=(u.hourangle, u.deg))
    idx = np.argmin((combined['RA']-coord.ra.deg)**2 + (combined['Dec']-coord.dec.deg)**2)
    print(np.sqrt((combined['RA']-coord.ra.deg)**2 + (combined['Dec']-coord.dec.deg)**2)[idx])
    # print(combined[idx])
    return radial_velocity, combined[idx]

if __name__ == '__main__':
    name = 'Gl 905'
    # combined = combine_Mann_Yee()
    # combined.write(f'{DIRNAME}/library/combined.csv', format='csv',)
    combined = Table.read(f'{DIRNAME}/library/combined.csv', format='csv',)
    rv, row = find_star(name, combined=combined)
    # All simbad IDs
    simbadres = Simbad.query_objectids(name)
    # simbadres.pprint_all()
    all_IDS = ''
    gaia_dr3 = ''
    for i in range(len(simbadres)):
        all_IDS += str(simbadres[i][0])
        all_IDS += '|'
        if str(simbadres[i][0])[:8] == 'Gaia DR3':
            gaia_dr3 = str(simbadres[i][0])
        if str(simbadres[i][0])[:2] == 'HD':
            print(str(simbadres[i][0]))
    items = [name.replace(' ', '_'), row['Source'], row['Teff'], row['e_Teff'], row['[Fe/H]'], row['e_[Fe/H]'],
             row['log(g)'], row['e_log(g)'], 'N/A', all_IDS, gaia_dr3, '', '', '', '', '', rv]
    print(*items, sep=',')



from scipy.special import jn
from scipy import integrate
import numpy as np
from numpy import fft
from scipy.ndimage.filters import convolve1d
# Functions from Yee et al. 2017. Specmatch emp

def rotmacro_ft(sigma, xi, vsini, u1=0.43, u2=0.31, intres=100):
    """Fourier Transform of Rotational-Macroturbulent Broadening Kernel
    Forumula B12 from Hirano et al. (2011)
    Args:
        sig (float): velocity frequency, s/km
        xi (float): radial/tangential macroturblence (see Gray), km/s
        vsini (float): projected surface velocity, km/s
        u1,u2 (float): quadradtic limbdarkening parameters
    Returns:
        array: Fourier transform of rot-macro profile at sig
    """ 
    
    tarr = np.outer( np.ones(sigma.shape[0]), np.linspace(0, 1, intres) )
    tarr = tarr.transpose()

    t1 = (
        (1 - u1 * ( 1 - np.sqrt(1 - tarr**2) ) - 
         u2 * ( 1 - np.sqrt( 1 - tarr**2 ) )**2) / ( 1 - u1 / 3 - u2 / 6 )
    )
    t2 = (
        ( np.exp( -np.pi**2 * xi**2 * sigma**2 * (1-tarr**2) ) + 
          np.exp( -np.pi**2 * xi**2 * sigma**2 * tarr**2) ) * 
        jn(0,2*np.pi*sigma*vsini*tarr) * tarr
    )

    m = t1 * t2
    kernel_ft = integrate.trapz(m,dx=np.mean(tarr[1:] - tarr[0:-1]),axis=0)
    return kernel_ft

def rotmacro(n, dv, xi, vsini, **kwargs):
    """Rotational-Macroturbulent Broadening Kernel
    Args:
        n (int): number of points over which to calculate kernel
        dv (float): spacing between points (km/s)
        xi (float): macroturbulence (km/s)
        vsini (float): projected rotational velocity
        **kwargs: passed to rotmacro_ft
    Returns:
        array: velocities
        array: broadening kernel (same size as varr)
    """
    nind = (n + 1) / 2 
    vmax = dv * nind
    varr = np.linspace( -vmax, vmax, n)

    # Fourier transform frequencies, for given velocity displacements
    sigarr = fft.fftfreq(n,d=dv)
    sigarr = sigarr[:nind]
    kernel_ft = rotmacro_ft(sigarr,xi,vsini,**kwargs)
    kernel_ft = np.hstack([ kernel_ft, kernel_ft[1:][::-1] ])
    kernel = fft.ifft(kernel_ft)
    kernel = fft.fftshift(kernel)
    kernel = kernel/np.sum(kernel)
    return varr,kernel


import numpy as np
from scipy.ndimage.filters import convolve1d

def rot(n, dv, vsini, u1=0.6):
    """Rotational Kernel
    Create a 1-d convolution kernel to broaden a spectrum from a rotating star
    
    Args:
        deltaV (float): spacing between kernel points (km/s)
        vsini (float): projecte rotational velocoity
        u1 (Optional[float]): linear limb-darkening
            coefficient. Default value of 0.6 is typical for
            photospheric lines. The specific intensity I at any angle
            theta from the specific intensity Icen at the center of
            the disk is given by:
  
            I = Icen*(1-epsilon*(1-cos(theta))
    Returns:
        array: The convolution kernel vector for the specified
            rotational velocity. kernel will always contain an odd number of 
            points
    Notes:
        Adapted from lsf_rotate.pro by Erik Petigura
    """
    nind = (n + 1) / 2 # number of independent points
    vmax = dv * nind
    varr = np.linspace(-vmax, vmax, n).astype(float)

    e1 = 2.0 * (1.0 - u1)
    e2 = np.pi * u1 / 2.0
    e3 = np.pi * (1.0 - u1 / 3.0)

    x = varr / vsini
    x1 = np.abs(1.0 - x**2)
    kernel = ( ( e1 * np.sqrt(x1) + e2 * x1 ) / e3).astype(float)
    kernel[np.abs(x) > 1] = 0 # kernel should be 0 for v > vsini 
    kernel /= kernel.sum() # normalize 
    return varr, kernel

def broaden(w,f,vsini,u1=0.6):
    """
    Applies a broadening kernel to the given spectrum (or error)
    Args:
        w - wavelength
        f - flux
        vsini (float): vsini to determine width of broadening
    Returns:
        broadened (Spectrum): Broadened spectrum
    """
    SPEED_OF_LIGHT = 2.99792e5
    dv = (w[1]-w[0])/w[0]*SPEED_OF_LIGHT
    n = 151     # fixed number of points in the kernel
    varr, kernel = rot(n, dv, vsini,u1=u1)
    # broadened = signal.fftconvolve(spec, kernel, mode='same')

    f = convolve1d(f, kernel)
    #err = convolve1d(err, kernel)

    return f



from setuptools import setup, find_packages

def readme():
    with open('README.md') as f:
        return f.read()

setup(
    name='neidspecmatch',
    version='0.1.2',
    description='Matching NEID Spectra',
    long_description=readme(),
    long_description_content_type='text/markdown',
    url='https://github.com/gummiks/hpfspecmatch/',
    author='Te Han, Gudmundur Stefansson (author of HPFSpecMatch)',
    author_email='gummiks@gmail.com',
    install_requires=[
        'numpy',
        'barycorrpy',
        'emcee',
        'lmfit',
        'neidspec',
        'crosscorr',
        'pyde',
        'astroquery',
        'glob2',
        'celerite',
        'numba',
    ],
    packages=find_packages(),
    license_files='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Topic :: Scientific/Engineering :: Astronomy',
    ],
    keywords='NEID Spectra Astronomy',
    include_package_data=True,
    zip_safe=False,
)


import os
import astropy.time
import matplotlib.pyplot as plt
import matplotlib
import numpy as np
import pickle
import re
import pandas as pd
import radvel
from astroquery.simbad import Simbad
import wget, zipfile
import neidspecmatch.config as config
norm_mean = lambda x: x/np.nanmean(x)

def pickle_dump(filename,obj):
    """
    write obj to to filename and save
    
    INPUT:
        filename - name to save pickle file (str)
        obj - object to write to file
    
    OUTPUT:
        obj saved as pickle file
        
    EXAMPLE:
        pickle_dump()
    
    """
    savefile = open(filename,"w")
    pickle.dump(obj, savefile)
    savefile.close()
    print("Saved to {}".format(filename))

def pickle_load(filename,python3=True):
    """
    load obj from filename
    
    INPUT:
        filename - name of saved pickle file (str)
        obj - object to write to file
    
    OUTPUT:
        load obj from pickle file
        
    EXAMPLE:
        pickle_load()
    """
    if python3:
        openfile = open(filename,"rb")
    return pickle.load(openfile,encoding='latin1')


def jd2datetime(times):
    """
    convert jd to iso utc date/time
    
    INPUT:
        times in jd (array)
    
    OUTPUT:
        date/time in utc (array)
        
    EXAMPLE:
        
    """
    return np.array([astropy.time.Time(time,format="jd",scale="utc").datetime for time in times])

def iso2jd(times):
    """
    convert iso utc to jd date/time
    
    INPUT:
        date/time in iso uts (array)
    
    OUTPUT:
        date/time in jd (array)
        
    EXAMPLE:
        
    """
    return np.array([astropy.time.Time(time,format="iso",scale="utc").jd for time in times])

def make_dir(dirname,verbose=True):
    """
    create new directory
    
    INPUT:
        dirname - name of sirectory (str)
        verbose - include print statements (bool)
    
    OUTPUT:
        creates folder named dirname
        
    EXAMPLE:
        make_dir("folder")
        
    """
    try:
        os.makedirs(dirname)
        if verbose==True: print("Created folder:",dirname)
    except OSError:
        if verbose==True: print(dirname,"already exists.")

def vac2air(wavelength,P,T,input_in_angstroms=True):
    """
    Convert vacuum wavelengths to air wavelengths

    INPUT:
        wavelength - in A if input_in_angstroms is True, else nm
        P - in Torr
        T - in Celsius

    OUTPUT:
        Wavelength in air in A if input_in_angstroms is True, else nm
    """
    if input_in_angstroms:
        nn = n_air(P,T,wavelength/10.)
    else:
        nn = n_air(P,T,wavelength)
    return wavelength/(nn+1.)

def n_air(P,T,wavelength):
    """
    The edlen equation for index of refraction of air with pressure
    
    INPUT:
        P - pressure in Torr
        T - Temperature in Celsius
        wavelength - wavelength in nm
        
    OUTPUT:
        (n-1)_tp - see equation 1, and 2 in REF below.
        
    REF:
        http://iopscience.iop.org/article/10.1088/0026-1394/30/3/004/pdf

    EXAMPLE:
        nn = n_air(763.,20.,500.)-n_air(760.,20.,500.)
        (nn/(nn + 1.))*3.e8
    """
    wavenum = 1000./wavelength # in 1/micron
    # refractivity is (n-1)_s for a standard air mixture
    refractivity = ( 8342.13 + 2406030./(130.-wavenum**2.) + 15997./(38.9-wavenum**2.))*1e-8
    return ((P*refractivity)/720.775) * ( (1.+P*(0.817-0.0133*T)*1e-6) / (1. + 0.0036610*T) )

def get_cmap_colors(cmap='jet',p=None,N=10):
    """
    get colormap instance
    
    INPUT:
        cmap - colormap instance
        p -
        N -
    
    OUTPUT:
        
        
    EXAMPLE:
        get_cmap_colors()
    """
    cm = plt.get_cmap(cmap)
    if p is None:
        return [cm(i) for i in np.linspace(0,1,N)]
    else:
        normalize = matplotlib.colors.Normalize(vmin=min(p), vmax=max(p))
        colors = [cm(normalize(value)) for value in p]
        return colors

def ax_apply_settings(ax,ticksize=None):
    """
    Apply axis settings that I keep applying
    """
    ax.minorticks_on()
    if ticksize is None:
        ticksize=12
    ax.tick_params(pad=3,labelsize=ticksize)
    ax.grid(lw=0.3,alpha=0.3)

def ax_add_colorbar(ax,p,cmap='jet',tick_width=1,tick_length=3,direction='out',pad=0.02,minorticks=False,*kw_args):
    """
    Add a colorbar to a plot (e.g., of lines)

    INPUT:
        ax - axis to put the colorbar
        p  - parameter that will be used for the scaling (min and max)
        cmap - 'jet', 'viridis'

    OUTPUT:
        cax - the colorbar object

    NOTES:
        also see here:
            import matplotlib.pyplot as plt
            sm = plt.cm.ScalarMappable(cmap=my_cmap, norm=plt.Normalize(vmin=0, vmax=1))
            # fake up the array of the scalar mappable. Urgh...
            sm._A = []
            plt.colorbar(sm)

    EXAMPLE:
        cmap = 'jet'
        colors = get_cmap_colors(N=len(bjds),cmap=cmap)
        fig, ax = plt.subplots(dpi=200)
        for i in range(len(bjds)):
            ax.plot(vin[i],ccf_sub[i],color=colors[i],lw=1)
        ax_apply_settings(ax,ticksize=14)
        ax.set_xlabel('Velocity [km/s]',fontsize=20)
        ax.set_ylabel('Relative Flux ',fontsize=20)
        cx = ax_add_colorbar(ax,obs_phases,cmap=cmap,pad=0.02)
        ax_apply_settings(cx)
        cx.axes.set_ylabel('Phase',fontsize=20,labelpad=0)


        cbar.set_clim(1.4,4.1)
        cbar.set_ticks([1.5,2.0,2.5,3.0,3.5,4.0])
    """
    cax, _ = matplotlib.colorbar.make_axes(ax,pad=pad,*kw_args)
    normalize = matplotlib.colors.Normalize(vmin=np.nanmin(p), vmax=np.nanmax(p))
    cm = plt.get_cmap(cmap)
    cbar = matplotlib.colorbar.ColorbarBase(cax, cmap=cm, norm=normalize)
    if minorticks:
        cax.minorticks_on()
    cax.axes.tick_params(width=tick_width,length=tick_length,direction=direction)
    return cax, cbar

def get_indices_of_items(arr,items):
    return np.where(pd.DataFrame(arr).isin(items))[0]


def remove_items_from_list(l,bad_items):
    ibad = np.where(pd.DataFrame(l).isin(bad_items))[0]
    return np.delete(l,ibad)

def savefigure(fig,savename,s1='{}',p1='',s2='{}',p2='',dpi=200):
    """
    Handy function to save figures and append suffixes to filenames
    
    EXAMPLE:
        savefigure(fig,'MASTER_FLATS/COMPARE_PLOTS/testing.png',s1='_o{}',p1=5,s2='_spi{}',p2=14)
    """
    fp = FilePath(savename)
    make_dir(fp.directory)
    fp.add_suffix(s1.format(p1))
    fp.add_suffix(s2.format(p2))
    fig.tight_layout()
    fig.savefig(fp._fullpath,dpi=dpi)
    print('Saved figure to: {}'.format(fp._fullpath))

def grep_date(string,intype="isot",outtype='iso'):
    """
    A function to extract date from string.

    INPUT:
        string: string
        intype: "isot" - 20181012T001823
                "iso"  - 20181012
        outtype: "iso" - iso
                 "datetime" - datetime

    OUTPUT:
        string with the date
    """
    if intype == "isot":
        date = re.findall(r'\d{8}T\d{6}',string)[0]
    elif intype == "iso":
        date = re.findall(r'\d{8}',string)[0]
    else:
        print("intype has to be 'isot' or 'iso'")
    if outtype == 'iso':
        return date
    elif outtype == 'datetime':
        return pd.to_datetime(date).to_pydatetime()
    else:
        print('outtype has to be "iso" or "datetime"')

def grep_dates(strings,intype="isot",outtype='iso'):
    """
    A function to extract date from strings

    INPUT:
        string: string
        intype: "isot" - 20181012T001823
                "iso"  - 20181012
        outtype: "iso" - iso
                 "datetime" - datetime

    OUTPUT:
        string with the date

    EXAMPLE:
        df = grep_dates(files,intype="isot",outtype='series')
        df['2018-06-26':].values
    """
    if outtype=='series':
        dates = [grep_date(i,intype=intype,outtype='datetime') for i in strings]
        return pd.Series(index=dates,data=strings)
    else:
        return [grep_date(i,intype,outtype) for i in strings]

def replace_dir(files,old,new):
    for i,f in enumerate(files):
        files[i] = files[i].replace(old,new)
    return files

def get_header_df(fitsfiles,keywords=["OBJECT","DATE-OBS"],verbose=True):
    """
    A function to read headers and returns a pandas dataframe

    INPUT:
    fitsfiles - a list of fitsfiles
    keywords - a list of header keywords to read

    OUTPUT:
    df - pandas dataframe with column names as the passed keywords
    """
    headers = []
    for i,name in enumerate(fitsfiles):
        if verbose: print(i,name)
        head = astropy.io.fits.getheader(name)
        values = [name]
        for key in keywords:
            values.append(head[key])
        headers.append(values)
    df_header = pd.DataFrame(headers,columns=["filename"]+keywords)
    return df_header

def plot_rv_model(bjd,RV,e_RV,P,T0,K,e,w,nbjd=None,nRV=None,ne_RV=None,title="",fig=None,ax=None,bx=None):
    """
    Plot RVs and the rv model on top. Also plot residuals
    """
    bjd_model = np.linspace(bjd[0]-2,bjd[-1]+2,10000)
    rv_model = get_rv_curve(bjd_model,P,T0,e=e,omega=w,K=K,plot=False)
    rv_obs = get_rv_curve(bjd,P,T0,e,w,K,plot=False)
    
    if nbjd is not None:
        rv_obs_bin = get_rv_curve(nbjd,P,T0,e,w,K,plot=False)

    # Plotting
    if fig is None and ax is None and bx is None:
        fig, (ax, bx) = plt.subplots(dpi=200,nrows=2,gridspec_kw={"height_ratios":[5,2]},figsize=(6,4),sharex=True)
    
    ax.errorbar(bjd,RV,e_RV,elinewidth=1,mew=0.5,capsize=5,marker="o",lw=0,markersize=6,alpha=0.5,
            label="Unbin, median errorbar={:0.1f}m/s".format(np.median(e_RV)))
    ax.plot(bjd_model,rv_model,label="Expected Orbit",lw=1,color="crimson")
    res = RV - rv_obs
    bx.errorbar(bjd,res,e_RV,elinewidth=1,mew=0.5,capsize=5,marker="o",lw=0,markersize=6,alpha=0.5,label='Residual: $\sigma$={:0.2f}m/s'.format(np.std(res)))
    
    if nbjd is not None:
        ax.errorbar(nbjd,nRV,ne_RV,elinewidth=1,mew=0.5,capsize=5,marker="h",lw=0,color="crimson",
            markersize=8,label="Bin, median errorbar={:0.1f}m/s".format(np.median(ne_RV)))
        bx.errorbar(nbjd,nRV-rv_obs_bin,ne_RV,elinewidth=1,mew=0.5,capsize=5,marker="h",lw=0,markersize=8,alpha=0.5,color="crimson")
    ax.legend(fontsize=8,loc="upper right")
    bx.legend(fontsize=8,loc="upper right")

    for xx in [ax,bx]:
        ax_apply_settings(xx,ticksize=12)
        xx.set_ylabel("RV [m/s]",fontsize=16)

    bx.set_xlabel("Date [UT]",labelpad=0,fontsize=16)
    ax.set_title(title)
    fig.tight_layout()
    fig.subplots_adjust(hspace=0.05)
    
def plot_rv_model_phased(bjd,RV,e_RV,P,T0,K,e,w,nbjd=None,nRV=None,ne_RV=None,title="",fig=None,ax=None,bx=None):
    """
    Plot RVs and the rv model on top. Also plot residuals
    """
    bjd_model = np.linspace(bjd[0]-2,bjd[-1]+2,10000)
    rv_model = get_rv_curve(bjd_model,P,T0,e=e,omega=w,K=K,plot=False)
    rv_obs = get_rv_curve(bjd,P,T0,e,w,K,plot=False)
    
    # Phases:
    df_phase = get_phases_sorted(bjd,P,T0).sort_values("time")
    df_phase_model = get_phases_sorted(bjd_model,P,T0,rvs=rv_model).sort_values("time")

    if nbjd is not None:
        rv_obs_bin = get_rv_curve(nbjd,P,T0,e,w,K,plot=False)
        df_phase_bin = get_phases_sorted(nbjd,P,T0).sort_values("time")

    # Plotting
    if fig is None and ax is None and bx is None:
        fig, (ax, bx) = plt.subplots(dpi=200,nrows=2,gridspec_kw={"height_ratios":[5,2]},figsize=(6,4),sharex=True)
    
    ax.errorbar(df_phase.phases,RV,e_RV,elinewidth=1,mew=0.5,capsize=5,marker="o",lw=0,markersize=6,alpha=0.5,
            label="Unbin, median errorbar={:0.1f}m/s".format(np.median(e_RV)))
    ax.plot(df_phase_model.phases,rv_model,label="Expected Orbit",lw=0,marker="o",markersize=2,color="crimson")
    res = RV - rv_obs
    bx.errorbar(df_phase.phases,res,e_RV,elinewidth=1,mew=0.5,capsize=5,marker="o",lw=0,markersize=6,alpha=0.5,label='Residual: $\sigma$={:0.2f}m/s'.format(np.std(res)))
    
    if nbjd is not None:
        ax.errorbar(df_phase_bin.phases,nRV,ne_RV,elinewidth=1,mew=0.5,capsize=5,marker="h",lw=0,color="crimson",
            markersize=8,label="Bin, median errorbar={:0.1f}m/s".format(np.median(ne_RV)),alpha=0.9)
        bx.errorbar(df_phase_bin.phases,nRV-rv_obs_bin,ne_RV,elinewidth=1,mew=0.5,capsize=5,marker="h",lw=0,markersize=8,alpha=0.9,color="crimson")
    ax.legend(fontsize=8,loc="upper right")
    bx.legend(fontsize=8,loc="upper right")

    for xx in [ax,bx]:
        ax_apply_settings(xx,ticksize=12)
        xx.set_ylabel("RV [m/s]",fontsize=16)

    bx.set_xlabel("Orbital phase",labelpad=0,fontsize=16)
    ax.set_title(title)
    fig.tight_layout()
    fig.subplots_adjust(hspace=0.05)

def get_phases_sorted(t, P, t0,rvs=None,rvs_err=None,sort=True,centered_on_0=True,tdur=None):
    """
    Get a sorted pandas dataframe of phases, times (and Rvs if supplied)
    
    INPUT:
    t  - times in jd
    P  - period in days
    t0 - time of periastron usually
    
    OUTPUT:
    df - pandas dataframe with columns:
     -- phases (sorted)
     -- time - time
     -- rvs  - if provided
    
    NOTES:
    Useful for RVs.    
    """
    phases = np.mod(t - t0,P)
    phases /= P
    df = pd.DataFrame(zip(phases,t),columns=['phases','time'])
    if rvs is not None:
        df['rvs'] = rvs
    if rvs_err is not None:
        df['rvs_err'] = rvs_err
    if centered_on_0:
        _p = df.phases.values
        m = df.phases.values > 0.5
        _p[m] = _p[m] - 1.
        df['phases'] = _p
    if tdur is not None:
        df["intransit"] = np.abs(df.phases) < tdur/(2.*P)
        print("Found {} in transit".format(len(df[df['intransit']])))
    if sort:
        df = df.sort_values('phases').reset_index(drop=True)
    return df

def get_rv_curve(times_jd,P,tc,e,omega,K,plot=True,ax=None,verbose=True,plot_tnow=True):
    """
    A function to plot an RV curve as a function of time (not phased)
    
    INPUT:
        times_jd: times in jd
        P: orbital period in days
        tc: transit center in jd
        e: eccentricity
        omega: periastron in degrees
        K: RV semi-amplitude in m/s
    
    OUTPUT:
        rv: array of RVs at times times_jd
    """
    t_peri = radvel.orbit.timetrans_to_timeperi(tc=tc,
                                                per=P,
                                                ecc=e,
                                                omega=np.deg2rad(omega))
    rvs = radvel.kepler.rv_drive(times_jd,[P,
                                            t_peri,
                                            e,
                                            np.deg2rad(omega),
                                            K])
    if verbose:
        print("Assuming:")
        print("P {}d".format(P))
        print("tc {}".format(tc))
        print("e {}".format(e))
        print("omega {}deg".format(omega))
        print("K {}m/s".format(K))
    if plot:
        if ax is None:
            fig, ax = plt.subplots(figsize=(12,8))
        times = jd2datetime(times_jd)
        ax.plot(times,rvs)
        ax.set_xlabel("Time")
        ax.set_ylabel("RV [m/s]")
        ax.grid(lw=0.5,alpha=0.3)
        ax.minorticks_on()
        if plot_tnow:
            t_now = Time(datetime.datetime.utcnow())
            ax.axvline(t_now.datetime,color="red",label="Time now")
        xlim = ax.get_xlim()
        ax.hlines(0,xlim[0],xlim[1],alpha=0.5,color="k",lw=1)
        for label in (ax.get_xticklabels()):
            label.set_fontsize(10)
    return rvs


def filter_simbadnames(l):
    """
    Filter SIMBAD names
    """
    _n = find_str_in_list(l,'DR2')
    if _n=='':
        _n = find_str_in_list(l,'DR1')
    if _n=='':
        _n = find_str_in_list(l,'HD')
    if _n=='':
        _n = find_str_in_list(l,'GJ')
    return _n

def get_simbad_object_names(name,as_str=False):
    """
    Get all Simbad names for an object
    """
    result_table = Simbad.query_objectids(name).to_pandas()
    names = result_table.ID.values.astype(str)
    if as_str:
        names = [i.replace(' ','_') for i in names]
        return '|'.join(names)
    else:
        return names

def find_str_in_list(l,string):
    """
    Return first element that matches a string in a list
    """
    for element in l:
        if string in element:
            return element
    return ''

def get_library(overwrite=False):
    """
    Download stellar library if it does not already exist

    INPUT:
        overwrite - if True, try redownloading and overwriting current library
    
    OUTPUT:
        saves downloaded zip file to outputdir folder
        
    EXAMPLE:
        neidspecmatch.utils.get_library()
    """

    if ((os.path.isdir(config.PATH_LIBRARY) is False) and (overwrite is False)) or (overwrite is True):
        make_dir(config.PATH_LIBRARY)
        
        print('Downloading library from: {}'.format(config.URL_LIBRARY))
        wget.download(config.URL_LIBRARY, config.PATH_LIBRARY_ZIPNAME)
        
        print('Extracting zip file')
        with zipfile.ZipFile(config.PATH_LIBRARY_ZIPNAME, 'r') as zip_ref:
            zip_ref.extractall(config.PATH_LIBRARY)  
        print('Extracting zip file complete')
        print('Deleting zip file')
        os.remove(config.PATH_LIBRARY_ZIPNAME)
        #shutil.rmtree('../library/__MACOSX')
        print('Download complete')
        
    else:
        print('Library already exists. Skipping download')
        print('Library path is: {}'.format(config.PATH_LIBRARY))
